@@ -730,7 +730,7 @@ void chat_printf(char *format, ...)
 
 	for (buddy = gtd->chat->next ; buddy ; buddy = buddy->next)
 	{
-		if (HAS_BIT(buddy->flags, CHAT_FLAG_FORWARD))
+		if (HAS_BIT(buddy->flags, CHAT_FLAG_FORWARD) && !HAS_BIT(buddy->flags, CHAT_FLAG_FORWARDALL))
 		{
 			chat_socket_printf(buddy, "%s", tmp);
 		}
@@ -850,7 +850,7 @@ int process_chat_input(struct chat_data *buddy)
 
 			RESTRING(buddy->name, name);
 
-			chat_socket_printf(buddy, "%c%s%s%c", CHAT_VERSION, "TinTin++ ", VERSION_NUM, CHAT_END_OF_COMMAND);
+			chat_socket_printf(buddy, "%c%s %s%c", CHAT_VERSION, CLIENT_NAME, CLIENT_VERSION, CHAT_END_OF_COMMAND);
 
 			get_chat_commands(buddy, sep, size - strlen(temp) - 1);
 
@@ -1003,7 +1003,7 @@ void get_chat_commands(struct chat_data *buddy, char *buf, int len)
 			case CHAT_VERSION:
 				if (*buddy->version == 0 && *txt != 0)
 				{
-					chat_socket_printf(buddy, "%c%s%s%c", CHAT_VERSION, "TinTin++ ", VERSION_NUM, CHAT_END_OF_COMMAND);
+					chat_socket_printf(buddy, "%c%s %s%c", CHAT_VERSION, CLIENT_NAME, CLIENT_VERSION, CHAT_END_OF_COMMAND);
 
 					RESTRING(buddy->version, txt);
 				}
@@ -1052,6 +1052,7 @@ void get_chat_commands(struct chat_data *buddy, char *buf, int len)
 			case CHAT_SNOOP_DATA:
 				SET_BIT(buddy->flags, CHAT_FLAG_FORWARDBY);
 				DEL_BIT(buddy->flags, CHAT_FLAG_FORWARD);
+				DEL_BIT(buddy->flags, CHAT_FLAG_FORWARDALL);
 				chat_receive_snoop_data(buddy, txt);
 				break;
 
@@ -1322,8 +1323,6 @@ void parse_requested_connections(struct chat_data *buddy, char *txt)
 		return;
 	}
 
-
-
 	ip[0] = 0;
 	port[0] = 0;
 	comma = txt;
