@@ -129,6 +129,7 @@ struct listnode *insert_node_list(struct listroot *root, char *ltext, char *rtex
 	node->left  = strdup(ltext);
 	node->right = strdup(rtext);
 	node->pr    = strdup(prtext);
+
 	node->group = HAS_BIT(root->flags, LIST_FLAG_CLASS) ? strdup(root->ses->group) : strdup("");
 
 	switch (root->type)
@@ -328,12 +329,13 @@ int locate_index_list(struct listroot *root, char *text, char *priority)
 
 int bsearch_alpha_list(struct listroot *root, char *text, int seek)
 {
-	long long bot, top, val, srt, toi;
+	long long bot, top, val;
+	double toi, toj, srt;
 
 	bot = 0;
 	top = root->used - 1;
 	val = top;
-	toi = (long long) get_number(root->ses, text);
+	toi = get_number(root->ses, text);
 
 	if (!seek && toi && (*text == '+' || *text == '-') && HAS_BIT(list_table[root->type].flags, LIST_FLAG_NEST))
 	{
@@ -353,7 +355,20 @@ int bsearch_alpha_list(struct listroot *root, char *text, int seek)
 
 	while (bot <= top)
 	{
-		srt = toi || atoll(root->list[val]->left) ? toi - atoll(root->list[val]->left) : strcmp(text, root->list[val]->left);
+		toj = get_number(root->ses, root->list[val]->left);
+
+		if (toi)
+		{
+			srt = toi - toj;
+		}
+		else if (toj)
+		{
+			srt = -1;
+		}
+		else
+		{
+			srt = strcmp(text, root->list[val]->left);
+		}
 
 		if (srt == 0)
 		{
@@ -384,7 +399,8 @@ int bsearch_alpha_list(struct listroot *root, char *text, int seek)
 
 int bsearch_priority_list(struct listroot *root, char *text, char *priority, int seek)
 {
-	int bot, top, val, srt;
+	int bot, top, val;
+	double srt;
 
 	bot = 0;
 	top = root->used - 1;
@@ -392,7 +408,7 @@ int bsearch_priority_list(struct listroot *root, char *text, char *priority, int
 
 	while (bot <= top)
 	{
-		srt = atoll(priority) - atoll(root->list[val]->pr);
+		srt = atof(priority) - atof(root->list[val]->pr);
 
 		if (!srt)
 		{
