@@ -30,46 +30,46 @@
 
 DO_COMMAND(do_event)
 {
-	char left[BUFFER_SIZE], right[BUFFER_SIZE];
+	char arg1[BUFFER_SIZE], arg2[BUFFER_SIZE];
 	struct listroot *root;
 	int cnt;
 
 	root = ses->list[LIST_EVENT];
 
-	arg = get_arg_in_braces(arg, left,  FALSE);
-	arg = get_arg_in_braces(arg, right, TRUE);
+	arg = sub_arg_in_braces(ses, arg, arg1, 0, SUB_VAR|SUB_FUN);
+	arg = get_arg_in_braces(arg, arg2, 1);
 
-	if (*left == 0)
+	if (*arg1 == 0)
 	{
 		tintin_header(ses, " EVENTS ");
 
 		for (cnt = 0 ; *event_table[cnt].name != 0 ; cnt++)
 		{
-			tintin_printf2(ses, "  [%-20s] [%s] %s", event_table[cnt].name, searchnode_list(ses->list[LIST_EVENT], event_table[cnt].name) ? "X" : " ", event_table[cnt].desc);
+			tintin_printf2(ses, "  [%-20s] [%s] %s", event_table[cnt].name, search_node_list(ses->list[LIST_EVENT], event_table[cnt].name) ? "X" : " ", event_table[cnt].desc);
 		}
 		tintin_header(ses, "");
 	}
-	else if (*right == 0)
+	else if (*arg2 == 0)
 	{
-		if (show_node_with_wild(ses, left, LIST_EVENT) == FALSE)
+		if (show_node_with_wild(ses, arg1, LIST_EVENT) == FALSE)
 		{
-			show_message(ses, LIST_ALIAS, "#EVENT: NO MATCH(ES) FOUND FOR {%s}.", left);
+			show_message(ses, LIST_ALIAS, "#EVENT: NO MATCH(ES) FOUND FOR {%s}.", arg1);
 		}
 	}
 	else
 	{
 		for (cnt = 0 ; *event_table[cnt].name != 0 ; cnt++)
 		{
-			if (!strncmp(event_table[cnt].name, left, strlen(event_table[cnt].name)))
+			if (!strncmp(event_table[cnt].name, arg1, strlen(event_table[cnt].name)))
 			{
-				show_message(ses, LIST_EVENT, "#EVENT {%s} HAS BEEN SET TO {%s}.", left, right);
+				show_message(ses, LIST_EVENT, "#EVENT {%s} HAS BEEN SET TO {%s}.", arg1, arg2);
 
-				updatenode_list(ses, left, right, "", LIST_EVENT);
+				update_node_list(ses->list[LIST_EVENT], arg1, arg2, "");
 
 				return ses;
 			}
 		}
-		tintin_printf(ses, "#EVENT {%s} IS NOT AN EXISTING EVENT.", capitalize(left));
+		tintin_printf(ses, "#EVENT {%s} IS NOT AN EXISTING EVENT.", capitalize(arg1));
 	}
 	return ses;
 }
@@ -83,8 +83,8 @@ DO_COMMAND(do_unevent)
 
 int check_all_events(struct session *ses, int args, int vars, char *fmt, ...)
 {
-	char buf[BUFFER_SIZE];
 	struct listnode *node;
+	char buf[BUFFER_SIZE];
 	va_list list;
 	int cnt;
 
@@ -92,31 +92,34 @@ int check_all_events(struct session *ses, int args, int vars, char *fmt, ...)
 
 	vsprintf(buf, fmt, list);
 
-	for (node = ses->list[LIST_EVENT]->f_node ; node ; node = node->next)
+	va_end(list); 
+
+	node = search_node_list(ses->list[LIST_EVENT], buf);
+
+	if (node)
 	{
-		if (!strcmp(node->left, buf))
+		va_start(list, fmt);
+
+		for (cnt = 0 ; cnt < args ; cnt++)
 		{
-			for (cnt = 0 ; cnt < args ; cnt++)
-			{
-				va_arg(list, char *);
-			}
-
-			for (cnt = 0 ; cnt < vars ; cnt++)
-			{
-				RESTRING(gtd->vars[cnt], va_arg(list, char *));
-			}
-
-			substitute(ses, node->right, buf, SUB_ARG|SUB_SEC);
-
-			script_driver(ses, LIST_EVENT, buf);
-
-			va_end(list);
-
-			return 1;
+			va_arg(list, char *);
 		}
+
+		for (cnt = 0 ; cnt < vars ; cnt++)
+		{
+			RESTRING(gtd->vars[cnt], va_arg(list, char *));
+		}
+
+		substitute(ses, node->right, buf, SUB_ARG);
+
+		script_driver(ses, LIST_EVENT, buf);
+
+		va_end(list);
+
+		return 1;
+	}
+	else
+	{
+		return 0;
 	}
-
-	va_end(list);
-
-	return 0;
 }
