@@ -30,22 +30,19 @@
 
 DO_COMMAND(do_gag)
 {
-	char left[BUFFER_SIZE];
-	struct listroot *root;
+	char arg1[BUFFER_SIZE];
 
-	root = ses->list[LIST_GAG];
+	arg = sub_arg_in_braces(ses, arg, arg1, 1, SUB_VAR|SUB_FUN);
 
-	arg = get_arg_in_braces(arg, left, TRUE);
-
-	if (*left == 0)
+	if (*arg1 == 0)
 	{
-		show_list(ses, root, LIST_GAG);
+		show_list(ses->list[LIST_GAG], 0);
 	}
 	else
 	{
-		updatenode_list(ses, left, "", "", LIST_GAG);
+		update_node_list(ses->list[LIST_GAG], arg1, "", "");
 
-		show_message(ses, LIST_GAG, "#OK. {%s} IS NOW GAGGED.", left);
+		show_message(ses, LIST_GAG, "#OK. {%s} IS NOW GAGGED.", arg1);
 	}
 	return ses;
 }
@@ -60,11 +57,11 @@ DO_COMMAND(do_ungag)
 
 void check_all_gags(struct session *ses, char *original, char *line)
 {
-	struct listnode *node;
+	struct listroot *root = ses->list[LIST_GAG];
 
-	for (node = ses->list[LIST_GAG]->f_node ; node ; node = node->next)
+	for (root->update = 0 ; root->update < root->used ; root->update++)
 	{
-		if (check_one_regexp(ses, node, line, original, 0))
+		if (check_one_regexp(ses, root->list[root->update], line, original, 0))
 		{
 			SET_BIT(ses->flags, SES_FLAG_GAG);
 
