@@ -151,12 +151,19 @@ struct help_type help_table[] =
 	},
 	{
 		"BUFFER",
-		"<178>Command<078>: #buffer <178>{<078>home<178>|<078>up<178>|<078>down<178>|<078>end<178>|<078>find<178>|<078>lock<178>|<078>write filename<178>|<078>info<178>}<078>\n"
+		"<178>Command<078>: #buffer <178>{<078>home<178>|<078>up<178>|<078>down<178>|<078>end<178>|<078>lock<178>|<078>clear<178>|<078>info<178>}<078>\n"
 		"         The buffer command allows you to add macros to scroll in case the\n"
-		"         default bindings do not work. The write option allows you to save\n"
-		"         the entire scrollback buffer to file. The lock options toggles the\n"
+		"         default bindings do not work. The lock options toggles the\n"
 		"         scrolling state of the screen.\n"
 		"\n"
+		"         #buffer {find} {number} {text} Jump to the first occasion of text\n"
+		"         or the given occasion by providing the optional number argument.\n"
+		"\n"
+		"         #buffer {get} {variable} {line} {range} Store the given line or\n"
+		"         range in the given variable.\n"
+		"\n"
+		"         #buffer {write} {filename} Write the buffer to the given file.\n"
+		"\n"
 		"<178>Example<078>: #macro {(press ctrl-v)(press F1)} {#buffer end}\n"
 		"         Associates F1 key to 'scroll buffer to its end' command.\n"
 	},
@@ -268,11 +275,12 @@ struct help_type help_table[] =
 		"\n"
 		"         Config options which aren't listed by default:\n"
 		"\n"
+		"         #CONFIG {BIG5}         {ON|OFF} BIG5 Chinese character handling.\n"
 		"         #CONFIG {CONVERT META} {ON|OFF} Shows color codes and key bindings.\n"
 		"         #CONFIG {COLOR PATCH}  {ON|OFF} Fixes color code usage on some muds.\n"
 		"         #CONFIG {DEBUG TELNET} {ON|OFF} Shows telnet negotiations y/n.\n"
+		"         #CONFIG {LOG LEVEL}  {LOW|HIGH} LOW logs mud output before triggers.\n"
 		"         #CONFIG {MCCP}         {ON|OFF} Enable or disable MCCP support.\n"
-		"         #CONFIG {LOG LEVEL}  {LOW|HIGH} LOW logs mud output before triggers\n"
 	},
 	{
 		"CONTINUE",
@@ -387,7 +395,6 @@ struct help_type help_table[] =
 		"\n"
 		"         \\a   will beep the terminal.\n"
 		"         \\c   will send a control character, \\ca for ctrl-a.\n"
-		"         \\d   will send a decimal character, \\d255 for example.\n"
 		"         \\e   will start an escape sequence.\n"
 		"         \\n   will send a line feed.\n"
 		"         \\r   will send a carriage return.\n"
@@ -410,21 +417,29 @@ struct help_type help_table[] =
 		"\n"
 		"         END OF PATH\n"
 		"         IAC <VAR> <VAR>\n"
+		"         IAC SB GMCP <MODULE> %0 data     %1 raw data\n"
 		"         IAC SB MSSP          %0 variable %1 value\n"
 		"         IAC SB MSDP          %0 variable %1 value\n"
+		"         IAC SB MSDP <VAR>    %1 value\n"
 		"         IAC SB NEW-ENVIRON   %0 variable %1 value\n"
 		"         IAC SB ZMP <VAR>     %0 value\n"
 		"         IAC SB <VAR>         %0 raw text %1 raw data\n"
-		"         MAP ENTER ROOM       %0 vnum\n"
-		"         MAP EXIT ROOM        %0 vnum\n"
+		"         MAP ENTER ROOM       %0 new vnum %1 old vnum\n"
+		"         MAP ENTER ROOM <VAR> %0 new vnum %1 old vnum\n"
+		"         MAP EXIT ROOM        %0 old vnum %1 new vnum\n"
+		"         MAP EXIT ROOM <VAR>  %0 old vnum %1 new vnum\n"
+		"         PROGRAM START        %0 client name %1 client version\n"
 		"         PROGRAM TERMINATION\n"
 		"         RECEIVED INPUT       %0 raw text\n"
 		"         RECEIVED LINE        %0 raw text %1 plain text\n"
+		"         SCREEN RESIZE        %0 cols %1 rows\n"
 		"         SEND OUTPUT          %0 raw output\n"
+		"         SESSION ACTIVATED    %0 name\n"
 		"         SESSION CONNECTED    %0 name %1 host %2 ip %3 port\n"
+		"         SESSION DEACTIVATED  %0 name\n"
 		"         SESSION DISCONNECTED %0 name %1 host %2 ip %3 port\n"
 		"\n"
-		"<178>Example<078>: #event {session connected} {#read mychar.tin}\n"
+		"<178>Example<078>: #event {SESSION CONNECTED} {#read mychar.tin}\n"
 		"\n"
 		"<178>Comment<078>: You can remove an event with the #unevent command.\n"
 	},
@@ -464,6 +479,7 @@ struct help_type help_table[] =
 		"         #format {test} {%a}   {number}  print corresponding ascii character\n"
 		"         #format {test} {%c}   {string}  use a highlight color name\n"
 		"         #format {test} {%d}   {number}  print a number with integer formatting\n"
+		"         #format {test} {%g}   {number}  perform thousand grouping on {number}\n"
 		"         #format {test} {%h}   {string}  turn text into a header line\n"
 		"         #format {test} {%l}   {string}  lowercase text\n"
 		"         #format {test} {%m}   {string}  perform mathematical calculation\n"
@@ -475,7 +491,6 @@ struct help_type help_table[] =
 		"         #format {test} {%u}   {string}  uppercase text\n"
 		"         #format {test} {%w}   {string}  wordwrap text\n"
 		"         #format {cols} {%C}         {}  store the screen width in {cols}\n"
-		"         #format {test} {%G}   {number}  perform thousand grouping on {number}\n"
 		"         #format {test} {%L}   {string}  store the string length in {test}\n"
 		"         #format {rows} {%R}         {}  store the screen height in {rows}\n"
 		"         #format {time} {%T}         {}  store the epoch time in {time}\n"
@@ -518,6 +533,33 @@ struct help_type help_table[] =
 		"\n"
 		"<178>Comment<078>: You can remove a gag with the #ungag command.\n"
 	},
+/*
+	{
+		"GREETING",
+		"<068>      #<068>##################################################################<068>#\n"
+		"<068>      #<078>                                                                  <068>#\n"
+		"<068>      #<078>                     T I N T I N + +   "CLIENT_VERSION"                     <068>#\n"
+		"<068>      #<078>                                                                  <068>#\n"
+		"<068>      #<078>           (<068>T<078>)he k(<068>I<078>)cki(<068>N<078>) (<068>T<078>)ickin d(<068>I<078>)kumud clie(<068>N<078>)t <068>          #\n"
+		"<068>      #<078>                                                                  <068>#\n"
+		"<068>      #<078>         Code by Peter Unold, Bill Reis, David A. Wagner,         <068>#\n"
+		"<068>      #<078>      Rob Ellsworth, Jeremy C. Jack, and Igor van den Hoven.      <068>#\n"
+		"<068>      #<078>                                                                  <068>#\n"
+		"<068>      #<078>                             1992, 2011                           <068>#\n"
+		"<068>      #<068>##################################################################<068>#<088>\n"
+	},
+*/
+	{
+		"GREETING",
+		"<068>      #<068>##################################################################<068>#\n"
+		"<068>      #<078>                     T I N T I N + +   "CLIENT_VERSION"                     <068>#\n"
+		"<068>      #<078>                                                                  <068>#\n"
+		"<068>      #<078>           (<068>T<078>)he k(<068>I<078>)cki(<068>N<078>) (<068>T<078>)ickin d(<068>I<078>)kumud clie(<068>N<078>)t <068>          #\n"
+		"<068>      #<078>                                                                  <068>#\n"
+		"<068>      #<078>         Code by Peter Unold, Bill Reis, David A. Wagner,         <068>#\n"
+		"<068>      #<078>      Rob Ellsworth, Jeremy C. Jack, and Igor van den Hoven.      <068>#\n"
+		"<068>      #<068>##################################################################<068>#<088>\n"
+	},
 	{
 		"GREP",
 		"<178>Command<078>: #grep <178>[<078>page<178>]<078> <178>{<078>search string<178>}<078>\n"
@@ -643,12 +685,18 @@ struct help_type help_table[] =
 		"LINE",
 		"<178>Command<078>: #line <178>{<078>gag<178>|<078>log<178>|<078>logverbatim<178>}<078> <178>{<078>argument<178>}<078>\n"
 		"\n"
-		"         #line log {filename} {[text]}          Log current or given line to file.\n"
+		"         #line log {filename} {[text]}          Log the current or given line to\n"
+		"                                                file.\n"
 		"         #line logverbatim {filename} {[text]}  Like log with no substitutions.\n"
 		"         #line gag                              Gag the next line.\n"
+		"         #line substitute {options} {argument}  Substitutes the given options:\n"
+		"                                                variables, functions, colors,\n"
+		"                                                escapes, secure, in the given\n"
+		"                                                argument next executes it as a\n"
+		"                                                command.\n"
 		"\n"
-		"         When using #line log and logging in html format use \\c< \\c> \\c& \\c\" to log a\n"
-		"         literal < > & and \".\n"
+		"         When using #line log and logging in html format use \\c< \\c> \\c& \\c\" to\n"
+		"         log a literal < > & and \".\n"
 	},
 	{
 		"LIST",
@@ -732,7 +780,9 @@ struct help_type help_table[] =
 		"         to use mapping for muds using 'ne' 'se' 'sw' 'nw' directions, you need\n"
 		"         to create a #pathdir for each direction.\n"
 		"\n"
-		"         #map color: Sets the map color of the current room you are in.\n"
+		"         #map at <location> <command>: Execute the command at the location.\n"
+		"\n"
+		"         #map color <field> value: Sets the map color for the given field.\n"
 		"\n"
 		"         #map create <size>: Creates a new map and room 1, required to start\n"
 		"                  mapping. Default size is 15000 rooms.\n"
@@ -741,16 +791,22 @@ struct help_type help_table[] =
 		"\n"
 		"         #map delete <direction>: Deletes the room in the given direction.\n"
 		"\n"
-		"         #map dig <direction>: Creates an exit in the given direction. If no\n"
-		"                  valid direction is given or no existing room is found a new\n"
-		"                  room is created. Useful for portal links and other alternative\n"
-		"                  forms of transportation.\n"
+		"         #map dig <direction> [new|<vnum>]: Creates an exit in the given\n"
+		"                  direction. If no valid direction is given or no existing room\n"
+		"                  is found a new room is created. Useful for portal links and\n"
+		"                  other alternative forms of transportation. If the 'new'\n"
+		"                  argument is provided all existing rooms are ignored. If a\n"
+		"                  room vnum is given as the second argument an exit will be\n"
+		"                  created to the given room vnum.\n"
 		"\n"
-		"         #map exit <exit name> <exit commands>: Sets the exit command. This is\n"
-		"                  useful if you are dealing with a closed door where you would\n"
-		"                  set the exit command with: '#map exit e {open east;e}'.\n"
-		"                  Keep in mind the exit command is only used when using\n"
-		"                  '#map find' and '#map walk'.\n"
+		"         #map exit <exit name> <COMMAND|DIRECTION|FLAG|NAME|VNUM> <argument>\n"
+		"                  Set the exit data. Useful with a closed door where you can\n"
+		"                  set the exit command: '#map exit e command {open east;e}'.\n"
+		"\n"
+		"         #map exitflag hide: When set, '#map' will not display the map beyond\n"
+		"                  this exit. When mapping overlapping areas or areas that aren't\n"
+		"                  build consistently you need this flag as well to stop\n"
+		"                  auto-linking, unless you use void rooms.\n"
 		"\n"
 		"         #map explore <direction>: Follows the direction until a dead end or an\n"
 		"                  intersection is found. The route is stored in #path and can\n"
@@ -785,7 +841,9 @@ struct help_type help_table[] =
 		"         #map goto <room name> [exits]: Takes you to the given room name, if\n"
 		"                  you provide exits those must match as well.\n"
 		"\n"
-		"         #map get <option> <variable>: Store a map value into a variable.\n"
+		"         #map get <option> <variable> [vnum]: Store a map value into a\n"
+		"                    variable, if no vnum is given the current room is used.\n"
+		"\n"
 		"         #map get roomexits <variable>: Store all room exits into variable.\n"
 		"\n"
 		"         #map info: Gives information about the map and room you are in.\n"
@@ -814,9 +872,11 @@ struct help_type help_table[] =
 		"\n"
 		"         #map list <location>: Lists all matching rooms and their distance.\n"
 		"\n"
-		"         #map map {<x>x<y>} {filename}: shows a map of surrounding rooms. The\n"
-		"                  {horizontal x vertical} argument i.e 80x25 is optional, and so\n"
-		"                  is the filename argument to log the map output to file.\n"
+		"         #map map {<x>x<y>} {filename} {a}: shows a map of surrounding rooms.\n"
+		"                  The {horizontal x vertical} argument i.e 80x25 is optional,\n"
+		"                  and so is the filename argument to log the map output to\n"
+		"                  file. The {a} argument causes data to be appended so it can\n"
+		"                  easily be tailed with the tail -f <file> shell command.\n"
 		"\n"
 		"         #map move <direction>: This does the same as an actual movement\n"
 		"                  command, updating your location on the map and creating new\n"
@@ -829,12 +889,15 @@ struct help_type help_table[] =
 		"\n"
 		"         #map read <filename>: Will load the given map file.\n"
 		"\n"
+		"         #map return: Returns you to your last known room after leaving the map\n"
+		"                  or loading a map.\n"
+		"\n"
 		"         #map roomflag avoid: When set, '#map find' will avoid a route leading\n"
 		"                  through that room. Useful when you want to avoid death traps.\n"
 		"         #map roomflag hide: When set, '#map' will not display the map beyond\n"
 		"                  this room. When mapping overlapping areas or areas that aren't\n"
 		"                  build consistently you need this flag as well to stop\n"
-		"                  auto-linking.\n"
+		"                  auto-linking, unless you use void rooms.\n"
 		"         #map roomflag leave: When entering a room with this flag, you will\n"
 		"                  automatically leave the map. Useful when set at the entrance\n"
 		"                  of an unmappable maze.\n"
@@ -845,12 +908,13 @@ struct help_type help_table[] =
 		"         #map roomflag static: When set the room will no longer be autolinked\n"
 		"                  when walking around. Useful for mapping mazes.\n"
 		"\n"
-		"         #map run [delay] <room name>: Calculates the shortest path to the\n"
+		"         #map run <room name> [delay]: Calculates the shortest path to the\n"
 		"                  destination and walks you there. The delay is optional and\n"
 		"                  requires using braces. Besides the room name a list of\n"
-		"                  exits can be provided as well for more precise matching.\n"
+		"                  exits can be provided for more precise matching.\n"
 		"\n"
-		"         #map set <option> <value>: Set a map value for your current room.\n"
+		"         #map set <option> <value> [vnum]: Set a map value for your current room,\n"
+		"                  or given room if a room vnum is provided.\n"
 		"\n"
 		"         #map travel <direction> <delay>: Follows the direction until a dead end\n"
 		"                  or an intersection is found. Use braces around the direction\n"
@@ -861,6 +925,8 @@ struct help_type help_table[] =
 		"                  they will be deleted, otherwise you'll simply move back a\n"
 		"                  room. Useful if you walked into a non existant direction.\n"
 		"\n"
+		"         #map uninsert <direction>: Exact opposite of the insert command.\n"
+		"\n"
 		"         #map unlink <direction> [both]: Will remove the exit, this isn't two\n"
 		"                  way so you can have the map properly display no exit rooms and\n"
 		"                  mazes.\n"
@@ -873,7 +939,7 @@ struct help_type help_table[] =
 		"MATH",
 		"<178>Command<078>: #math <178>{<078>variable<178>}<078> <178>{<078>expression<178>}<078>\n"
 		"\n"
-		"         Performs math functions and stored the result in a variable.  The math\n"
+		"         Performs math operations and stores the result in a variable.  The math\n"
 		"         follows a C-like precedence, as follows, with the top of the list\n"
 		"         having the highest priority.\n"
 		"\n"
@@ -904,7 +970,8 @@ struct help_type help_table[] =
 		"\n"
 		"         True is any non-zero number, and False is zero.  Parentheses () have\n"
 		"         highest precedence, so inside the () is always evaluated first.\n"
-		"         Strings must be enclosed in \" \" and are evaluated in alphabetic order.\n"
+		"         Strings must be enclosed in \" \" and use regex with == and !=,\n"
+		"         in the case of <= and >= the alphabetic order is compared.\n"
 		"\n"
 		"<178>Example<078>: #math {heals} {$mana / 40}\n"
 		"         Assuming there is a variable $mana, divides its value by 40 and stores\n"
@@ -916,7 +983,7 @@ struct help_type help_table[] =
 		"         level.  The above will modify that variable after every kill, showing\n"
 		"         the amount still needed.\n"
 		"\n"
-		"<178>Example<078>: #action {%0 tells %1}\\\n"
+		"<178>Example<078>: #action {%0 tells %1}\n"
 		"           {#if {\"\%0\" == \"Bubba\" && $afk} {reply I'm away, my friend.}}\n"
 		"         When you are away from keyboard, it will only reply to your friend.\n"
 	},
@@ -962,13 +1029,13 @@ struct help_type help_table[] =
 		"         #path del:  Will delete the last move of the path.\n"
 		"         #path end:  Movement commands are no longer added to the path.\n"
 		"         #path ins:  Add the given argument to the path.\n"
-		"         #path load: Load the given alias as the new path.\n"
+		"         #path load: Load the given variable as the new path.\n"
 		"         #path map:  Display the so far created path.\n"
 		"         #path new:  Movement commands will be added to the path.\n"
 		"         #path run:  Enter all the steps of the current path, with\n"
 		"                     an optional given delay.\n"
-		"         #path save: Save the created path to an alias. You must specify whether\n"
-		"                     you want to save the path forward or backward.\n"
+		"         #path save: Save the created path to a variable. You must specify\n"
+		"                     whether you want to save the path forward or backward.\n"
 		"         #path walk: Take one step forward of the current path. You can walk\n"
 		"                     backwards using #path walk backwards.\n"
 		"\n"
@@ -1373,10 +1440,11 @@ struct help_type help_table[] =
 	},
 	{
 		"ZAP",
-		"<178>Command<078>: #zap\n"
+		"<178>Command<078>: #zap {[session]}\n"
 		"\n"
 		"         Kill your current session.  If there is no current session, it will\n"
-		"         cause the program to terminate.\n"
+		"         cause the program to terminate. If you provide an argument it'll zap\n"
+		"         the given session instead.\n"
 	},
 	{
 		"",
@@ -1394,8 +1462,6 @@ DO_COMMAND(do_help)
 
 	if (*left == 0)
 	{
-		tintin_header(ses, " HELP LIST ");
-
 		for (cnt = add[0] = 0 ; *help_table[cnt].name != 0 ; cnt++)
 		{
 			if ((int) strlen(add) + 19 > ses->cols)
@@ -1406,8 +1472,6 @@ DO_COMMAND(do_help)
 			cat_sprintf(add, "%19s", help_table[cnt].name);
 		}
 		tintin_puts2(ses, add);
-
-		tintin_header(ses, "");
 	}
 	else
 	{
@@ -1419,10 +1483,6 @@ DO_COMMAND(do_help)
 			{
 				substitute(ses, help_table[cnt].text, buf, SUB_COL);
 
-				tintin_header(ses, " %s ", help_table[cnt].name);
-
-				tintin_puts2(ses, "");
-
 				pto = buf;
 
 				while (*pto)
@@ -1439,7 +1499,7 @@ DO_COMMAND(do_help)
 
 					pto = ptf;
 				}
-				tintin_puts2(ses, "");
+//				tintin_puts2(ses, "");
 
 				found = TRUE;
 
