@@ -29,8 +29,6 @@
 
 void process_input(void)
 {
-	int flags;
-
 	if (HAS_BIT(gtd->ses->telopts, TELOPT_FLAG_SGA)	&& !HAS_BIT(gtd->ses->telopts, TELOPT_FLAG_ECHO))
 	{
 		read_key();
@@ -56,16 +54,16 @@ void process_input(void)
 
 	if (HAS_BIT(gtd->ses->telopts, TELOPT_FLAG_ECHO))
 	{
-		add_line_history(gts, gtd->input_buf);
+		add_line_history(gtd->ses, gtd->input_buf);
 	}
 
 	if (HAS_BIT(gtd->ses->telopts, TELOPT_FLAG_ECHO))
 	{
-		echo_command(gtd->ses, gtd->input_buf, FALSE);
+		echo_command(gtd->ses, gtd->input_buf);
 	}
 	else
 	{
-		echo_command(gtd->ses, "", FALSE);		
+		echo_command(gtd->ses, "");
 	}
 
 	if (gtd->ses->scroll_line != -1)
@@ -73,14 +71,8 @@ void process_input(void)
 		buffer_end(gtd->ses, "");
 	}
 
-	flags = TINTIN_FLAG_USERCOMMAND | (HAS_BIT(gtd->ses->flags, SES_FLAG_VERBATIM) ? TINTIN_FLAG_VERBATIM : 0);
-
-	SET_BIT(gtd->flags, flags);
-
 	gtd->ses = script_driver(gtd->ses, -1, gtd->input_buf);
 
-	DEL_BIT(gtd->flags, flags);
-
 	check_all_events(gtd->ses, 0, 1, "RECEIVED INPUT", gtd->input_buf);
 
 	if (IS_SPLIT(gtd->ses))
@@ -118,9 +110,9 @@ void read_line()
 		match = 0;
 		root  = gtd->ses->list[LIST_MACRO];
 
-		for (node = root->f_node ; node ; node = root->update)
+		for (root->update = 0 ; root->update < root->used ; root->update++)
 		{
-			root->update = node->next;
+			node = root->list[root->update];
 
 			if (!strcmp(gtd->macro_buf, node->pr))
 			{
@@ -206,6 +198,10 @@ void read_line()
 
 				cursor_check_line("");
 
+				DEL_BIT(gtd->flags, TINTIN_FLAG_HISTORYBROWSE);
+
+				kill_list(gtd->ses->list[LIST_TABCYCLE]);
+
 				if (HAS_BIT(gtd->flags, TINTIN_FLAG_HISTORYSEARCH))
 				{
 					cursor_history_find("");
@@ -245,12 +241,12 @@ void read_key(void)
 	if (!HAS_BIT(gtd->ses->flags, SES_FLAG_CONVERTMETA))
 	{
 		match = 0;
-	
+
 		root  = gtd->ses->list[LIST_MACRO];
 
-		for (node = root->f_node ; node ; node = root->update)
+		for (root->update = 0 ; root->update < root->used ; root->update++)
 		{
-			root->update = node->next;
+			node = root->list[root->update];
 
 			if (!strcmp(gtd->macro_buf, node->pr))
 			{
@@ -349,21 +345,47 @@ void convert_meta(char *input, char *output)
 				pti++;
 				break;
 
+			case '\a':
+				*pto++ = '\\';
+				*pto++ = 'a';
+				pti++;
+				break;
+
+			case '\b':
+				*pto++ = '\\';
+				*pto++ = 'b';
+				pti++;
+				break;
+
+			case '\t':
+				*pto++ = '\\';
+				*pto++ = 't';
+				pti++;
+				break;
+
 			case '\r':
 				*pto++ = '\\';
 				*pto++ = 'r';
 				pti++;
 				break;
+
 			case '\n':
 				*pto++ = *pti++;
 				break;
 
 			default:
-				if (*pti < 32)
+				if (*pti > 0 && *pti < 32)
 				{
 					*pto++ = '\\';
 					*pto++ = 'c';
-					*pto++ = 'a' + *pti - 1;
+					if (*pti <= 26)
+					{
+						*pto++ = 'a' + *pti - 1;
+					}
+					else
+					{
+						*pto++ = 'A' + *pti - 1;
+					}
 					pti++;
 					break;
 				}
@@ -408,6 +430,11 @@ void unconvert_meta(char *input, char *output)
 						pti += 3;
 						break;
 
+					case 'a':
+						*pto++  = '\a';
+						pti += 2;
+						break;
+
 					case 'b':
 						*pto++  = 127;
 						pti    += 2;
@@ -452,7 +479,7 @@ void unconvert_meta(char *input, char *output)
 	Currenly only used in split mode.
 */
 
-void echo_command(struct session *ses, char *line, int newline)
+void echo_command(struct session *ses, char *line)
 {
 	char buffer[STRING_SIZE], result[STRING_SIZE];
 
@@ -465,32 +492,19 @@ void echo_command(struct session *ses, char *line, int newline)
 		sprintf(buffer, "%s", line);
 	}
 
-	if (newline == TRUE)
+	/*
+		Deal with pending output
+	*/
+
+	if (ses->more_output[0])
 	{
-		if (ses->more_output[0] == 0)
+		if (ses->check_output)
 		{
-			return;
+			strcpy(result, ses->more_output);
+			ses->more_output[0] = 0;
+
+			process_mud_output(ses, result, FALSE);
 		}
-		ses->check_output = 0;
-
-		if (HAS_BIT(ses->flags, SES_FLAG_ECHOCOMMAND))
-		{
-			SET_BIT(ses->flags, SES_FLAG_SCROLLSTOP);
-
-			if (HAS_BIT(ses->flags, SES_FLAG_SPLIT))
-			{
-				tintin_printf2(ses, "%s%s", ses->more_output, buffer);
-			}
-			else
-			{
-				tintin_printf2(ses, "%s", buffer);
-			}
-
-			DEL_BIT(ses->flags, SES_FLAG_SCROLLSTOP);
-		}
-		add_line_buffer(ses, buffer, -1);
-
-		return;
 	}
 
 	DEL_BIT(ses->telopts, TELOPT_FLAG_PROMPT);
@@ -538,3 +552,19 @@ void input_printf(char *format, ...)
 
 	printf("%s", buf);
 }
+
+void modified_input(void)
+{
+	kill_list(gtd->ses->list[LIST_TABCYCLE]);
+
+	if (HAS_BIT(gtd->flags, TINTIN_FLAG_HISTORYSEARCH))
+	{
+		cursor_history_find("");
+	}
+
+	if (HAS_BIT(gtd->flags, TINTIN_FLAG_HISTORYBROWSE))
+	{
+		DEL_BIT(gtd->flags, TINTIN_FLAG_HISTORYBROWSE);
+	}
+
+}
