@@ -119,7 +119,7 @@ void write_html_header(FILE *fp)
 		"<html>\n"
 		"<head>\n"
 		"<meta http-equiv='content-type' content='text/html; charset=iso-8859-1'>\n"
-		"<meta name='description' content='Generated by TinTin++ "VERSION_NUM" - http://tintin.sourceforge.net'>\n"
+		"<meta name='description' content='Generated by TinTin++ "CLIENT_VERSION" - http://tintin.sourceforge.net'>\n"
 		"<style type='text/css'>\n"
 		"{\n\tfont-family: Courier;\n\tfont-size: 10pt;\n}\n"
 		".d30{ color: #000; } .l30{ color: #555; } .b40{ background-color: #000; } .b50{ background-color: #555 }\n"
@@ -176,7 +176,7 @@ void vt102_to_html(struct session *ses, char *txt, char *out)
 						cnt = -1;
 						pti += 1 + strlen(tmp);
 
-						if (HAS_BIT(vtc, COL_256))
+						if (HAS_BIT(vtc, COL_256) && (HAS_BIT(vtc, COL_XTF) || HAS_BIT(vtc, COL_XTB)))
 						{
 							if (HAS_BIT(vtc, COL_XTF))
 							{
@@ -228,20 +228,20 @@ void vt102_to_html(struct session *ses, char *txt, char *out)
 									bgc = 48;
 									break;
 								default:
-									if (atoi(tmp) >= 40 && atoi(tmp) < 50)
+									if (atoi(tmp) / 10 == 4)
 									{
 										bgc = atoi(tmp);
 									}
-									if (atoi(tmp) >= 100 && atoi(tmp) < 110)
+									if (atoi(tmp) / 10 == 10)
 									{
 										bgc = atoi(tmp) - 50;
 									}
 
-									if (atoi(tmp) >= 30 && atoi(tmp) < 40)
+									if (atoi(tmp) / 10 == 3)
 									{
 										fgc = atoi(tmp);
 									}
-									if (atoi(tmp) >= 90 && atoi(tmp) < 100)
+									if (atoi(tmp) / 10 == 9)
 									{
 										SET_BIT(vtc, COL_BLD);
 
