@@ -37,6 +37,10 @@ struct tintin_data *gtd;
 
 void pipe_handler(int signal)
 {
+	restore_terminal();
+
+	clean_screen(gtd->ses);
+
 	tintin_printf(NULL, "broken_pipe: dumping stack");
 
 	dump_stack();
@@ -75,6 +79,24 @@ void winch_handler(int signal)
 
 void abort_handler(int signal)
 {
+	static char crashed = FALSE;
+
+	if (crashed)
+	{
+		exit(-1);
+	}
+	crashed = TRUE;
+
+	restore_terminal();
+
+	clean_screen(gtd->ses);
+
+	dump_stack();
+
+	fflush(NULL);
+
+	exit(-1);
+
 	if (gtd->ses->connect_retry > utime())
 	{
 		gtd->ses->connect_retry = 0;
@@ -95,7 +117,7 @@ void suspend_handler(int signal)
 
 	fflush(stdout);
 
-	reset_terminal();
+	restore_terminal();
 
 	kill(0, SIGSTOP);
 
@@ -116,7 +138,7 @@ void trap_handler(int signal)
 	}
 	crashed = TRUE;
 
-	reset_terminal();
+	restore_terminal();
 
 	clean_screen(gtd->ses);
 
@@ -135,12 +157,12 @@ void trap_handler(int signal)
 
 int main(int argc, char **argv)
 {
+	int greeting = TRUE;
+
 	#ifdef SOCKS
 		SOCKSinit(argv[0]);
 	#endif
 
-	init_tintin();
-
 	if (signal(SIGTERM, trap_handler) == BADSIG)
 	{
 		syserr("signal SIGTERM");
@@ -198,7 +220,26 @@ int main(int argc, char **argv)
 	{
 		int c;
 
-		while ((c = getopt(argc, argv, "e: h r: t: v")) != EOF)
+		while ((c = getopt(argc, argv, "e: G h r: t: v")) != EOF)
+		{
+			if (c == 'G')
+			{
+				greeting = FALSE;
+			}
+		}
+
+		optind = 1;
+	}
+
+	init_tintin(greeting);
+
+	if (argc > 1)
+	{
+		int c;
+
+		optind = 1;
+
+		while ((c = getopt(argc, argv, "e: G h r: t: v")) != EOF)
 		{
 			switch (c)
 			{
@@ -206,15 +247,20 @@ int main(int argc, char **argv)
 					gtd->ses = script_driver(gtd->ses, -1, optarg);
 					break;
 
+				case 'G':
+					break;
+
 				case 'h':
 					tintin_printf(NULL, "Usage: %s [OPTION]... [FILE]...", argv[0]);
 					tintin_printf(NULL, "");
 					tintin_printf(NULL, "  -e  Execute given command.");
+					tintin_printf(NULL, "  -G  Don't show the greeting screen.");
+					tintin_printf(NULL, "  -h  This help section.");
 					tintin_printf(NULL, "  -r  Read given file.");
 					tintin_printf(NULL, "  -t  Set given title.");
 					tintin_printf(NULL, "  -v  Enable verbose mode.");
 
-					reset_terminal();
+					restore_terminal();
 					exit(1);
 					break;
 
@@ -235,12 +281,14 @@ int main(int argc, char **argv)
 					break;
 			}
 		}
-		
+
 		if (argv[optind] != NULL)
 		{
 			gtd->ses = do_read(gtd->ses, argv[optind]);
 		}
-	} 
+	}
+	check_all_events(gts, 0, 2, "PROGRAM START", CLIENT_NAME, CLIENT_VERSION);
+	check_all_events(gts, 0, 2, "SCREEN RESIZE", ntos(gts->cols), ntos(gts->rows));
 
 	mainloop();
 
@@ -248,7 +296,7 @@ int main(int argc, char **argv)
 }
 
 
-void init_tintin(void)
+void init_tintin(int greeting)
 {
 	int ref, index;
 
@@ -309,6 +357,8 @@ void init_tintin(void)
 
 	printf("\033=\033[?1h");
 
+	gts->input_level++;
+
 	do_configure(gts, "{AUTO TAB}         {5000}");
 	do_configure(gts, "{BUFFER SIZE}     {20000}");
 	do_configure(gts, "{COMMAND COLOR}   {<078>}");
@@ -316,7 +366,7 @@ void init_tintin(void)
 	do_configure(gts, "{CONNECT RETRY}      {15}");
 	do_configure(gts, "{HISTORY SIZE}     {1000}");
 	do_configure(gts, "{LOG}               {RAW}");
-	do_configure(gts, "{PACKET PATCH}     {0.01}");
+	do_configure(gts, "{PACKET PATCH}     {0.00}");
 	do_configure(gts, "{REPEAT CHAR}         {!}");
 	do_configure(gts, "{REPEAT ENTER}      {OFF}");
 	do_configure(gts, "{SCROLL LOCK}        {ON}");
@@ -327,6 +377,8 @@ void init_tintin(void)
 	do_configure(gts, "{VERBOSE}           {OFF}");
 	do_configure(gts, "{WORDWRAP}           {ON}");
 
+	gts->input_level--;
+
 	insert_node_list(gts->list[LIST_PATHDIR],  "n",  "s",  "1");
 	insert_node_list(gts->list[LIST_PATHDIR],  "e",  "w",  "2");
 	insert_node_list(gts->list[LIST_PATHDIR],  "s",  "n",  "4");
@@ -341,27 +393,12 @@ void init_tintin(void)
 
 	init_terminal();
 
-	do_showme(gts, "");
-	do_showme(gts, "<068>      #<068>###################################################################<068>#");
-	do_showme(gts, "<068>      #<078>                                                                   <068>#");
-	do_showme(gts, "<068>      #<078>                     T I N T I N + +   "VERSION_NUM"                      <068>#");
-	do_showme(gts, "<068>      #<078>                                                                   <068>#");
-	do_showme(gts, "<068>      #<078>           (<068>T<078>)he k(<068>I<078>)cki(<068>N<078>) (<068>T<078>)ickin d(<068>I<078>)kumud clie(<068>N<078>)t <068>           #");
-	do_showme(gts, "<068>      #<078>                                                                   <068>#");
-	do_showme(gts, "<068>      #<078>         Code by Peter Unold, Bill Reis, David A. Wagner,          <068>#");
-	do_showme(gts, "<068>      #<078>      Rob Ellsworth, Jeremy C. Jack, and Igor van den Hoven.       <068>#");
-	do_showme(gts, "<068>      #<078>                                                                   <068>#");
-	do_showme(gts, "<068>      #<078>                             1992, 2009                            <068>#");
+	if (greeting)
+	{
+		do_advertise(gts, "");
 
-//	do_showme(gts, "<068>      #<078>                 Original TINTIN code by Peter Unold               <068>#");
-//	do_showme(gts, "<068>      #<078>       new code by Bill Reiss, David A. Wagner, Rob Ellsworth,     <068>#");
-//	do_showme(gts, "<068>      #<078>                 Jeremy C. Jack, Igor van den Hoven                <068>#");
-//	do_showme(gts, "<068>      #<078>                             1992, 2009                            <068>#");
-	do_showme(gts, "<068>      #<078>                                                                   <068>#");
-//	do_showme(gts, "<068>      #<078>                               "VERSION_NUM"                              <068>#");
-//	do_showme(gts, "<068>      #<078>                                                                   <068>#");
-	do_showme(gts, "<068>      #<068>###################################################################<068>#<088>");
-	do_showme(gts, "");
+		do_help(gts, "GREETING");
+	}
 }
 
 
@@ -369,6 +406,8 @@ void quitmsg(char *message)
 {
 	struct session *ses;
 
+	SET_BIT(gtd->flags, TINTIN_FLAG_TERMINATE);
+
 	while ((ses = gts->next) != NULL)
 	{
 		cleanup_session(ses);
@@ -391,7 +430,7 @@ void quitmsg(char *message)
 		history_write(gts, filename);
 	}
 */
-	reset_terminal();
+	restore_terminal();
 
 	clean_screen(gts);
 
