@@ -276,7 +276,7 @@ int mathexp_tokenize(struct session *ses, char *str)
 					case '"':
 						if (pta != buf3)
 						{
-							show_message(NULL, -1, "MATH EXP: \" FOUND INSIDE A NUMBER");
+							show_message(ses, -1, "MATH EXP: \" FOUND INSIDE A NUMBER");
 							return FALSE;
 						}
 						*pta++ = *pti++;
@@ -286,7 +286,7 @@ int mathexp_tokenize(struct session *ses, char *str)
 					case '(':
 						if (pta != buf3)
 						{
-							show_message(NULL, -1, "#MATH EXP: PARANTESES FOUND INSIDE A NUMBER");
+							show_message(ses, -1, "#MATH EXP: PARANTESES FOUND INSIDE A NUMBER");
 							return FALSE;
 						}
 						*pta++ = *pti++;
@@ -302,7 +302,8 @@ int mathexp_tokenize(struct session *ses, char *str)
 						*pta++ = *pti++;
 						if (point >= 0)
 						{
-							show_message(NULL, -1, "#MATH EXP: MORE THAN ONE POINT FOUND INSIDE A NUMBER");
+							show_message(ses, -1, "#MATH EXP: MORE THAN ONE POINT FOUND INSIDE A NUMBER");
+							precision = 0;
 							return FALSE;
 						}
 						point++;
@@ -473,7 +474,7 @@ int mathexp_tokenize(struct session *ses, char *str)
 						break;
 
 					default:
-//						show_message(NULL, -1, "#MATH EXP: UNKNOWN OPERATOR: %c", *pti);
+//						show_message(ses, -1, "#MATH EXP: UNKNOWN OPERATOR: %c", *pti);
 						return FALSE;
 				}
 				break;
@@ -482,7 +483,7 @@ int mathexp_tokenize(struct session *ses, char *str)
 
 	if (level != 0)
 	{
-		show_message(NULL, -1, "#MATH EXP: UNMATCHED PARENTHESES, LEVEL: %d", level);
+		show_message(ses, -1, "#MATH EXP: UNMATCHED PARENTHESES, LEVEL: %d", level);
 		return FALSE;
 	}
 
@@ -557,7 +558,7 @@ void mathexp_compute(struct session *ses, struct link_data *node)
 		case 'd':
 			if (tintoi(node->next->str3) <= 0)
 			{
-				show_message(NULL, -1, "#MATHEXP: INVALID DICE: %s", node->next->str3);
+				show_message(ses, -1, "#MATHEXP: INVALID DICE: %s", node->next->str3);
 				value = 0;
 			}
 			else
@@ -571,7 +572,7 @@ void mathexp_compute(struct session *ses, struct link_data *node)
 		case '/':
 			if (tintoi(node->next->str3) == 0)
 			{
-				show_message(NULL, -1, "#MATH ERROR: DIVISION ZERO.");
+				show_message(ses, -1, "#MATH ERROR: DIVISION ZERO.");
 				value = tintoi(node->prev->str3);
 			}
 			else
@@ -589,7 +590,7 @@ void mathexp_compute(struct session *ses, struct link_data *node)
 		case '%':
 			if (tintoi(node->next->str3) == 0)
 			{
-				show_message(NULL, -1, "#MATH ERROR: MODULO ZERO.");
+				show_message(ses, -1, "#MATH ERROR: MODULO ZERO.");
 				value = tintoi(node->prev->str3);
 			}
 			else
@@ -673,7 +674,7 @@ void mathexp_compute(struct session *ses, struct link_data *node)
 			value = (tineval(ses, node->prev->str3, node->next->str3) == 0);
 			break;
 		default:
-			show_message(NULL, -1, "#COMPUTE EXP: UNKNOWN OPERATOR: %c%c", node->str3[0], node->str3[1]);
+			show_message(ses, -1, "#COMPUTE EXP: UNKNOWN OPERATOR: %c%c", node->str3[0], node->str3[1]);
 			value = 0;
 			break;
 	}
@@ -702,6 +703,48 @@ void mathexp_compute(struct session *ses, struct link_data *node)
 
 double tintoi(char *str)
 {
+	char *ptr = str;
+
+	if (*ptr == 0)
+	{
+		return 0;
+	}
+
+	switch (*ptr)
+	{
+		case '!':
+		case '~':
+		case '+':
+		case '-':
+			ptr++;
+			break;
+	}
+
+	while (*ptr)
+	{
+		if (!isdigit((int) *ptr))
+		{
+			if (*ptr != '.')
+			{
+				return 0;
+			}
+			ptr++;
+
+			while (*ptr)
+			{
+				if (!isdigit((int) *ptr))
+				{
+					return 0;
+				}
+				ptr++;
+			}
+		}
+		else
+		{
+			ptr++;
+		}
+	}
+
 	switch (str[0])
 	{
 		case '!':
@@ -717,14 +760,7 @@ double tintoi(char *str)
 			return -atof(&str[1]);
 
 		default:
-			if (isdigit(str[0]))
-			{
-				return atof(str);
-			}
-			else
-			{
-				return 0;
-			}
+			return atof(str);
 	}
 }
 
