@@ -41,7 +41,7 @@ struct session *parse_input(struct session *ses, char *input)
 		return ses;
 	}
 
-	if (VERBATIM)
+	if (VERBATIM(ses))
 	{
 		line = (char *) malloc(BUFFER_SIZE);
 
@@ -76,7 +76,7 @@ struct session *parse_input(struct session *ses, char *input)
 	{
 		input = space_out(input);
 
-		input = get_arg_all(input, line);
+		input = get_arg_all(input, line, GET_ALL);
 
 		if (parse_command(ses, line))
 		{
@@ -95,10 +95,11 @@ struct session *parse_input(struct session *ses, char *input)
 			write_mud(ses, line, SUB_VAR|SUB_FUN|SUB_ESC|SUB_EOL);
 		}
 
-		if (*input == ';')
+		if (*input == COMMAND_SEPARATOR)
 		{
 			input++;
 		}
+
 	}
 
 	free(line);
@@ -186,7 +187,7 @@ void process_speedwalk(struct session *ses, char *input)
 
 	for (dir[1] = 0 ; *input ; input++)
 	{
-		if (isdigit(*input))
+		if (isdigit((int) *input))
 		{
 			sscanf(input, "%d%c", &cnt, dir);
 
@@ -268,7 +269,7 @@ struct session *parse_tintin_command(struct session *ses, char *input)
 */
 
 
-char *get_arg_all(char *string, char *result)
+char *get_arg_all(char *string, char *result, int flag)
 {
 	char *pto, *pti;
 	int nest = 0;
@@ -306,7 +307,7 @@ char *get_arg_all(char *string, char *result)
 		{
 			*pto++ = *pti++;
 		}
-		else if (*pti == COMMAND_SEPARATOR && nest == 0 && !VERBATIM)
+		else if (*pti == COMMAND_SEPARATOR && nest == 0 && !HAS_BIT(flag, GET_VBT))
 		{
 			break;
 		}
@@ -491,7 +492,7 @@ char *get_arg_stop_spaces(char *string, char *result, int flag)
 		{
 			break;
 		}
-		else if (isspace(*pti) && nest == 0)
+		else if (isspace((int) *pti) && nest == 0)
 		{
 			pti++;
 			break;
@@ -525,7 +526,7 @@ char *get_arg_stop_spaces(char *string, char *result, int flag)
 
 char *space_out(char *string)
 {
-	while (isspace(*string))
+	while (isspace((int) *string))
 	{
 		string++;
 	}
@@ -702,18 +703,23 @@ void write_mud(struct session *ses, char *command, int flags)
 
 	size = substitute(ses, command, output, flags);
 
-	write_line_mud(ses, output, size);
-
-	if (ses->map && ses->map->in_room)
-	{
-		follow_map(ses, command);
-	}
-
 	if (HAS_BIT(ses->flags, SES_FLAG_MAPPING))
 	{
-		check_insert_path(command, ses);
+		if (ses->map == NULL || ses->map->nofollow == 0)
+		{
+			check_insert_path(command, ses);
+		}
 	}
 
+	if (ses->map && ses->map->in_room && ses->map->nofollow == 0)
+	{
+		if (follow_map(ses, command))
+		{
+			return;
+		}
+	}
+
+	write_line_mud(ses, output, size);
 }
 
 
