@@ -138,12 +138,7 @@ struct session *newactive_session(void)
 	}
 	else
 	{
-		gtd->ses = gts;
-		dirty_screen(gtd->ses);
-
-		tintin_printf(NULL, "");
-
-		tintin_printf(NULL, "#THERE'S NO ACTIVE SESSION NOW.");
+		activate_session(gts);
 	}
 	pop_call();
 	return gtd->ses;
@@ -151,12 +146,16 @@ struct session *newactive_session(void)
 
 struct session *activate_session(struct session *ses)
 {
+	check_all_events(gtd->ses, 0, 1, "SESSION DEACTIVATED", gtd->ses->name);
+
 	gtd->ses = ses;
 
 	dirty_screen(ses);
 
 	tintin_printf(ses, "#SESSION '%s' ACTIVATED.", ses->name);
 
+	check_all_events(ses, 0, 1, "SESSION ACTIVATED", ses->name);
+
 	return ses;
 }
 
@@ -172,6 +171,12 @@ struct session *new_session(struct session *ses, char *name, char *address, int
 
 	push_call("new_session(%p,%p,%p,%d)",ses,name,address,desc);
 
+	if (HAS_BIT(gtd->flags, TINTIN_FLAG_TERMINATE))
+	{
+		pop_call();
+		return ses;
+	}
+
 	address = get_arg_in_braces(address, host, FALSE);
 	address = get_arg_in_braces(address, port, FALSE);
 
@@ -229,15 +234,27 @@ struct session *new_session(struct session *ses, char *name, char *address, int
 		newsession->list[cnt] = copy_list(newsession, gts->list[cnt], cnt);
 	}
 
-	init_screen_size(newsession);
+	newsession->rows          = gts->rows;
+	newsession->cols          = gts->cols;
+	newsession->top_row       = gts->top_row;
+	newsession->bot_row       = gts->bot_row;
 
 	init_buffer(newsession, gts->scroll_max);
 
+	gtd->ses = newsession;
+
 	dirty_screen(newsession);
 
-	tintin_printf2(ses, "#Trying to connect to %s port %s.", newsession->host, newsession->port);
+	if (desc)
+	{
+		tintin_printf2(ses, "#TRYING TO LAUNCH '%s' RUNNING '%s'.", newsession->name, newsession->host);
+	}
+	else
+	{
+		tintin_printf2(ses, "#TRYING TO CONNECT '%s' TO '%s' PORT '%s'.", newsession->name, newsession->host, newsession->port);
+	}
+
 
-	gtd->ses = newsession;
 
 	if (desc == 0)
 	{
@@ -403,6 +420,12 @@ void dispose_session(struct session *ses)
 		delete_map(ses);
 	}
 
+	if (ses->mccp)
+	{
+		inflateEnd(ses->mccp);
+		free(ses->mccp);
+	}
+
 	init_buffer(ses, 0);
 
 	free(ses->name);
