@@ -37,6 +37,8 @@ char iac_do_sga[]             = { IAC, DO,    TELOPT_SGA             };
 char iac_will_sga[]           = { IAC, WILL,  TELOPT_SGA             };
 
 char iac_sb_tspeed[]          = { IAC, SB,    TELOPT_TSPEED, ENV_SEND, IAC, SE };
+
+char iac_dont_ttype[]         = { IAC, DONT,  TELOPT_TTYPE           };
 char iac_sb_ttype[]           = { IAC, SB,    TELOPT_TTYPE,  ENV_SEND, IAC, SE  };
 
 
@@ -50,6 +52,8 @@ char iac_sb_mssp[]            = { IAC, SB,    TELOPT_MSSP            };
 
 char iac_sb_msdp[]            = { IAC, SB,    TELOPT_MSDP            };
 
+char iac_sb_gmcp[]            = { IAC, SB,    TELOPT_GMCP            };
+
 char iac_sb_new_environ[]     = { IAC, SB,    TELOPT_NEW_ENVIRON     };
 
 char iac_sb_zmp[]             = { IAC, SB,    TELOPT_ZMP             };
@@ -82,8 +86,10 @@ struct iac_type iac_table [] =
 	{   3,  iac_will_mssp,        &send_do_mssp            },
 	{   3,  iac_sb_mssp,          &recv_sb_mssp            },
 	{   3,  iac_sb_msdp,          &recv_sb_msdp            },
+	{   3,  iac_sb_gmcp,          &recv_sb_gmcp            },
 	{   3,  iac_sb_new_environ,   &recv_sb_new_environ     },
 	{   6,  iac_sb_tspeed,        &recv_sb_tspeed          },
+	{   3,  iac_dont_ttype,       &recv_dont_ttype         },
 	{   6,  iac_sb_ttype,         &recv_sb_ttype           },
 	{   3,  iac_sb_zmp,           &recv_sb_zmp             },
 	{   5,  iac_sb_mccp1,         &init_mccp               },
@@ -121,6 +127,8 @@ int translate_telopts(struct session *ses, unsigned char *src, int cplen)
 		return 0;
 	}
 
+	push_call("translate_telopts(%p,%p,%d)",ses,src,cplen);
+
 	if (ses->mccp)
 	{
 		ses->mccp->next_in   = src;
@@ -149,11 +157,12 @@ int translate_telopts(struct session *ses, unsigned char *src, int cplen)
 				break;
 
 			case Z_STREAM_END:
-				cplen = ses->mccp->next_out - gtd->mccp_buf;
-				cpsrc = gtd->mccp_buf;
+				tintin_printf2(ses, "");
+				tintin_printf2(ses, "#COMPRESSION END, DISABLING MCCP.");
+
+				cpsrc = src + (cplen - ses->mccp->avail_in);
+				cplen = ses->mccp->avail_in;
 
-				tintin_puts2(ses, "");
-				tintin_puts2(ses, "#COMPRESSION END, DISABLING MCCP.");
 				inflateEnd(ses->mccp);
 				free(ses->mccp);
 				ses->mccp = NULL;
@@ -176,7 +185,7 @@ int translate_telopts(struct session *ses, unsigned char *src, int cplen)
 		cpsrc = src;
 	}
 
-	if (ses->read_len + cplen >= ses->read_max)
+ 	if (ses->read_len + cplen >= ses->read_max)
 	{
 		ses->read_max = ses->read_len + cplen + 1000;
 		ses->read_buf = (unsigned char *) realloc(ses->read_buf, ses->read_max);
@@ -275,6 +284,7 @@ int translate_telopts(struct session *ses, unsigned char *src, int cplen)
 
 					if (iac_table[cnt].func == init_mccp)
 					{
+						pop_call();
 						return translate_telopts(ses, cpsrc + skip, cplen - skip);
 					}
 					break;
@@ -348,6 +358,8 @@ int translate_telopts(struct session *ses, unsigned char *src, int cplen)
 						if (cplen > 2)
 						{
 							check_all_events(ses, 2, 0, "IAC %s %s", TELCMD(cpsrc[1]), telopt_table[cpsrc[2]].name);
+
+							DEL_BIT(ses->telopt_flag[cpsrc[2] / 32], 1 << cpsrc[2] % 32);
 						}
 						skip = 3;
 						break;
@@ -379,6 +391,7 @@ int translate_telopts(struct session *ses, unsigned char *src, int cplen)
 
 				gtd->mud_output_buf[gtd->mud_output_len] = 0;
 
+				pop_call();
 				return cplen;
 			}
 		}
@@ -411,7 +424,7 @@ int translate_telopts(struct session *ses, unsigned char *src, int cplen)
 					continue;
 
 				case '\r':
-					if (cplen < 2 || cpsrc[1] != 0)
+					if (cplen > 1 && cpsrc[1] == '\n')
 					{
 						cpsrc++;
 						cplen--;
@@ -461,6 +474,7 @@ int translate_telopts(struct session *ses, unsigned char *src, int cplen)
 
 	gtd->mud_output_buf[gtd->mud_output_len] = 0;
 
+	pop_call();
 	return 0;
 }
 
@@ -507,11 +521,22 @@ int mark_prompt(struct session *ses, int cplen, unsigned char *cpsrc)
 	TTYPE
 */
 
+int recv_dont_ttype(struct session *ses, int cplen, unsigned char *cpsrc)
+{
+	if (!check_all_events(ses, 0, 0, "IAC DONT TERMINAL TYPE"))
+	{
+		DEL_BIT(ses->telopts, TELOPT_FLAG_TTYPE);
+
+		DEL_BIT(ses->telopt_flag[cpsrc[2] / 32], 1 << cpsrc[2] % 32);
+	}
+	return 3;
+}
+
 int recv_sb_ttype(struct session *ses, int cplen, unsigned char *cpsrc)
 {
 	if (!check_all_events(ses, 0, 1, "IAC SB TERMINAL TYPE", ntos(cpsrc[3])))
-	{        
-		if (HAS_BIT(ses->telopts, TELOPT_FLAG_TSPEED) && atoi(ses->port) == TELNET_PORT && getenv("TERM"))
+	{
+		if (HAS_BIT(ses->telopts, TELOPT_FLAG_TTYPE) && getenv("TERM"))
 		{
 			socket_printf(ses, 6 + strlen(getenv("TERM")), "%c%c%c%c%s%c%c", IAC, SB, TELOPT_TTYPE, 0, getenv("TERM"), IAC, SE);
 
@@ -522,6 +547,8 @@ int recv_sb_ttype(struct session *ses, int cplen, unsigned char *cpsrc)
 			socket_printf(ses, 14, "%c%c%c%c%s%c%c", IAC, SB, TELOPT_TTYPE, 0, "TINTIN++", IAC, SE);
 
 			telopt_debug(ses, "SENT IAC SB TTYPE %s", "TINTIN++");
+
+			SET_BIT(ses->telopts, TELOPT_FLAG_TTYPE);
 		}
 	}
 	return 6;
@@ -750,13 +777,17 @@ int recv_sb_mssp(struct session *ses, int cplen, unsigned char *src)
 	MSDP (Mud Server Data Protocol)
 */
 
+#define MSDP_STATE_NEW 0
+#define MSDP_STATE_VAR 1
+#define MSDP_STATE_VAL 2
+
 int recv_sb_msdp(struct session *ses, int cplen, unsigned char *src)
 {
-	char var[BUFFER_SIZE], val[BUFFER_SIZE];
-	char *pto;
-	int i;
+	char buf[BUFFER_SIZE], var[BUFFER_SIZE], val[BUFFER_SIZE];
+	char *pto, *ptarr[1000];
+	int i, nest = 0, state[1000];
 
-	var[0] = val[0] = i = 0;
+	var[0] = val[0] = i = state[nest] = 0;
 
 	if (skip_sb(ses, cplen, src) > cplen)
 	{
@@ -767,30 +798,147 @@ int recv_sb_msdp(struct session *ses, int cplen, unsigned char *src)
 	{
 		switch (src[i])
 		{
-			case MSSP_VAR:
+			case MSDP_VAR:
 				i++;
-				pto = var;
+				pto = buf;
 
-				while (i < cplen && src[i] >= 3 && src[i] != IAC)
+				while (i < cplen && src[i] > 4 && src[i] != IAC)
 				{
 					*pto++ = src[i++];
 				}
 				*pto = 0;
+
+				substitute(ses, buf, var, SUB_SEC);
+
+				state[nest] = 0;
 				break;
 
-			case MSSP_VAL:
-				i++;
+			case MSDP_VAL:
 				pto = val;
 
-				while (i < cplen && src[i] >= 3 && src[i] != IAC)
+				while (i < cplen)
 				{
-					*pto++ = src[i++];
+					switch (src[i])
+					{
+						case MSDP_VAR:
+							if (state[nest] >= 2)
+							{
+								*pto++ = '}';
+							}
+
+							if (nest == 0)
+							{
+								goto end_val;
+							}
+
+							if (state[nest] >= 1)
+							{
+								*pto++ = '}';
+							}
+							*pto++ = '{';
+
+							state[nest] = 0;
+							break;
+
+						case MSDP_VAL:
+							if (state[nest] == 0)
+							{
+								if (nest)
+								{
+									*pto++ = '}';
+									*pto++ = '{';
+								}
+								ptarr[nest] = pto;
+								state[nest]++;
+							}
+							else if (state[nest] == 1)
+							{
+								sprintf(buf, "{1}{%.*s}{2}{", (int) (pto - ptarr[nest]), ptarr[nest]);
+								sprintf(ptarr[nest], "%s", buf);
+								pto += 9;
+								state[nest]++;
+							}
+							else
+							{
+								pto += sprintf(pto, "}{%d}{", ++state[nest]);
+							}
+							break;
+
+						case MSDP_OPEN:
+							if (++nest > 1)
+							{
+								*pto++ = '{';
+							}
+							state[nest] = 0;
+							break;
+
+						case MSDP_CLOSE:
+							if (state[nest] >= 2)
+							{
+								*pto++ = '}';
+							}
+
+							if (state[nest] >= 1)
+							{
+								*pto++ = '}';
+							}
+
+							if (--nest > 0)
+							{
+								*pto++ = '}';
+							}
+
+							break;
+
+						case IAC:
+							if (state[nest] >= 2)
+							{
+								*pto++ = '}';
+							}
+							goto end_val;
+
+						case '\\':
+							*pto++ = '\\';
+							*pto++ = '\\';
+							break;
+
+						case '{':
+							*pto++ = '\\';
+							*pto++ = 'x';
+							*pto++ = '7';
+							*pto++ = 'B';
+							break;
+
+						case '}':
+							*pto++ = '\\';
+							*pto++ = 'x';
+							*pto++ = '7';
+							*pto++ = 'D';
+							break;
+
+						case COMMAND_SEPARATOR:
+							*pto++ = '\\';
+							*pto++ = COMMAND_SEPARATOR;
+							break;
+
+						default:
+							*pto++ = src[i];
+							break;
+					}
+					i++;
 				}
+
+				end_val:
+
 				*pto = 0;
 
-				telopt_debug(ses, "IAC SB MSDP (VAR %-20s VAL %s)", var, val);
+				telopt_debug(ses, "IAC SB MSDP VAR %-20s VAL %s", var, val);
+
+				check_all_events(ses, 1, 2, "IAC SB MSDP %s", var, var, val);
 
 				check_all_events(ses, 0, 2, "IAC SB MSDP", var, val);
+
+				state[nest] = 0;
 				break;
 
 			default:
@@ -802,6 +950,7 @@ int recv_sb_msdp(struct session *ses, int cplen, unsigned char *src)
 	telopt_debug(ses, "IAC SB MSDP IAC SE");
 
 	check_all_events(ses, 0, 0, "IAC SB MSDP IAC SE");
+
 	return UMIN(i + 1, cplen);
 }
 
@@ -811,7 +960,7 @@ int recv_sb_msdp(struct session *ses, int cplen, unsigned char *src)
 
 int recv_sb_new_environ(struct session *ses, int cplen, unsigned char *src)
 {
-	char var[BUFFER_SIZE], val[BUFFER_SIZE];
+	char buf[BUFFER_SIZE], var[BUFFER_SIZE], val[BUFFER_SIZE];
 	char *pto;
 	int i, x;
 
@@ -835,7 +984,7 @@ int recv_sb_new_environ(struct session *ses, int cplen, unsigned char *src)
 			case ENV_USR:
 				x = src[i];
 				i++;
-				pto = var;
+				pto = buf;
 
 				while (i < cplen && src[i] >= 4 && src[i] != IAC)
 				{
@@ -843,6 +992,8 @@ int recv_sb_new_environ(struct session *ses, int cplen, unsigned char *src)
 				}
 				*pto = 0;
 
+				substitute(ses, buf, var, SUB_SEC);
+
 				if (src[3] == ENV_SEND)
 				{
 					telopt_debug(ses, "IAC SB NEW-ENVIRON SEND %s", x ? "VAR" : "USR");
@@ -853,7 +1004,7 @@ int recv_sb_new_environ(struct session *ses, int cplen, unsigned char *src)
 
 			case ENV_VAL:
 				i++;
-				pto = val;
+				pto = buf;
 
 				while (i < cplen && src[i] >= 3 && src[i] != IAC)
 				{
@@ -861,6 +1012,8 @@ int recv_sb_new_environ(struct session *ses, int cplen, unsigned char *src)
 				}
 				*pto = 0;
 
+				substitute(ses, buf, val, SUB_SEC);
+
 				telopt_debug(ses, "IAC SB NEW-ENVIRON %s %s", src[3] ? "IS" : "INFO", x ? "VAR" : "USR");
 
 				check_all_events(ses, 2, 2, "IAC SB NEW-ENVIRON %s %s", src[i] ? "IS" : "INFO", x ? "VAR" : "USR", var, val);
@@ -890,7 +1043,7 @@ int recv_sb_new_environ(struct session *ses, int cplen, unsigned char *src)
 
 int recv_sb_zmp(struct session *ses, int cplen, unsigned char *src)
 {
-	char var[BUFFER_SIZE], val[BUFFER_SIZE];
+	char buf[BUFFER_SIZE], var[BUFFER_SIZE], val[BUFFER_SIZE];
 	char *pto;
 	int i, x;
 
@@ -912,7 +1065,7 @@ int recv_sb_zmp(struct session *ses, int cplen, unsigned char *src)
 				break;
 
 			default:
-				pto = x ? val : var;
+				pto = buf;
 
 				while (i < cplen && src[i])
 				{
@@ -920,6 +1073,8 @@ int recv_sb_zmp(struct session *ses, int cplen, unsigned char *src)
 				}
 				*pto = src[i++];
 
+				substitute(ses, buf, x ? val : var, SUB_SEC);
+
 				if (x++)
 				{
 					telopt_debug(ses, "IAC SB ZMP %s", var);
@@ -938,6 +1093,218 @@ int recv_sb_zmp(struct session *ses, int cplen, unsigned char *src)
 }
 
 
+int recv_sb_gmcp(struct session *ses, int cplen, unsigned char *src)
+{
+	char mod[BUFFER_SIZE], val[BUFFER_SIZE], json[BUFFER_SIZE], *pto;
+	int i, state[1000], nest, type;
+
+	push_call("recv_sb_gmcp(%p,%d,%p)",ses,cplen,src);
+
+	if (skip_sb(ses, cplen, src) > cplen)
+	{
+		pop_call();
+		return cplen + 1;
+	}
+
+	mod[0] = val[0] = state[0] = nest = type = 0;
+
+	i = 3;
+
+	pto = mod;
+
+	// space out
+
+	while (i < cplen && src[i] == ' ')
+	{
+		i++;
+	}
+
+	// grab module
+
+	while (i < cplen && src[i] != IAC)
+	{
+		if (src[i] == ' ' || src[i] == '{' || src[i] == '[')
+		{
+			break;
+		}
+		*pto++ = src[i++];
+	}
+
+	*pto = 0;
+
+	// parse JSON content
+
+	pto = val;
+
+	while (i < cplen && src[i] != IAC)
+	{
+		switch (src[i])
+		{
+			case ' ':
+				i++;
+				break;
+
+			case '{':
+				if (nest != 0)
+				{
+					*pto++ = '{';
+				}
+				i++;
+				state[++nest] = 0;
+				break;
+
+			case '}':
+				nest--;
+				i++;
+				if (nest != 0)
+				{
+					*pto++ = '}';
+				}
+				break;
+
+			case '[':
+				if (nest != 0)
+				{
+					*pto++ = '{';
+				}
+				i++;
+				state[++nest] = 1;
+				pto += sprintf(pto, "{%d}", state[nest]);
+				break;
+
+			case ']':
+				nest--;
+				i++;
+				if (nest != 0)
+				{
+					*pto++ = '}';
+				}
+				break;
+
+			case ':':
+				i++;
+				break;
+
+			case ',':
+				i++;
+				if (state[nest])
+				{
+					pto += sprintf(pto, "{%d}", ++state[nest]);
+				}
+				break;
+
+			case '"':
+				i++;
+				if (nest)
+				{
+					*pto++ = '{';
+				}
+				type = 1;
+
+				while (i < cplen && src[i] != IAC && type == 1)
+				{
+					switch (src[i])
+					{
+						case '\\':
+							*pto++ = src[i++];
+							if (i < cplen && src[i] != IAC)
+							{
+								*pto++ = src[i++];
+							}
+							break;
+
+						case '"':
+							i++;
+							type = 0;
+							break;
+
+						case '{':
+							i++;
+							*pto++ = '\\';
+							*pto++ = 'x';
+							*pto++ = '7';
+							*pto++ = 'B';
+							break;
+
+						case '}':
+							i++;
+							*pto++ = '\\';
+							*pto++ = 'x';
+							*pto++ = '7';
+							*pto++ = 'D';
+							break;
+
+						case COMMAND_SEPARATOR:
+							i++;
+							*pto++ = '\\';
+							*pto++ = COMMAND_SEPARATOR;
+							break;
+
+						default:
+							*pto++ = src[i++];
+							break;
+					}
+				}
+
+				if (nest)
+				{
+					*pto++ = '}';
+				}
+				break;
+
+			default:
+				if (nest)
+				{
+					*pto++ = '{';
+				}
+
+				type = 1;
+
+				while (i < cplen && src[i] != IAC && type == 1)
+				{
+					switch (src[i])
+					{
+						case '}':
+						case ']':
+						case ',':
+						case ':':
+							type = 0;
+							break;
+
+						case ' ':
+							i++;
+							break;
+
+						default:
+							*pto++ = src[i++];
+							break;
+					}
+				}
+
+				if (nest)
+				{
+					*pto++ = '}';
+				}
+				break;
+		}
+	}
+	*pto = 0;
+
+	while (i < cplen && src[i] != SE)
+	{
+		i++;
+	}
+
+	sprintf(json, "%.*s", i - 4, src + 3);
+
+	telopt_debug(ses, "IAC SB GMCP %s IAC SE", mod);
+
+	check_all_events(ses, 1, 2, "IAC SB GMCP %s IAC SE", mod, val, json);
+
+	pop_call();
+	return UMIN(i + 1, cplen);
+}
+
 /*
 	MCCP
 */
@@ -1032,6 +1399,8 @@ int skip_sb(struct session *ses, int cplen, unsigned char *cpsrc)
 		}
 	}
 
+	telopt_debug(ses, "SKIP SB (%d)", cplen);
+
 	return cplen + 1;
 }
 
