@@ -35,7 +35,7 @@ void printline(struct session *ses, char *str, int prompt)
 	push_call("printline(%p,%p,%d)",ses,str,prompt);
 
 	if (ses->scroll_line != -1 && HAS_BIT(ses->flags, SES_FLAG_SCROLLLOCK))
- 	{
+	{
 		pop_call();
 		return;
 	}
@@ -79,13 +79,13 @@ void printline(struct session *ses, char *str, int prompt)
 
 int word_wrap(struct session *ses, char *textin, char *textout, int display)
 {
-	char *pti, *pto, *lis, *los;
+	char *pti, *pto, *lis, *los, *chi, *cho;
 	int skip = 0, cnt = 0;
 
 	push_call("word_wrap(%s,%p,%p)",ses->name, textin,textout);
 
-	pti = lis = textin;
-	pto = los = textout;
+	pti = chi = lis = textin;
+	pto = cho = los = textout;
 
 	ses->cur_col = 1;
 
@@ -138,17 +138,42 @@ int word_wrap(struct session *ses, char *textin, char *textout, int display)
 					los = pto;
 					lis = pti;
 				}
-				else
+				else if (lis != chi) // infinite VT loop detection
 				{
 					pto = los;
 					*pto++ = '\n';
-					pti = lis;
+					pti = chi = lis;
+					pti++;
+				}
+				else if (los != cho)
+				{
+					pto = cho = los;
+					pto++;
+					pti = chi = lis;
 					pti++;
 				}
 			}
 		}
 		else
 		{
+#ifdef BIG5
+			if (*pti & 128 && pti[1] != 0)
+			{
+				*pto++ = *pti++;
+			}
+#endif
+
+#ifdef UTF8
+			if (*pti & 128 && *pti & 64)
+			{
+				skip = (*pti & 16) ? 3 : (*pti & 32) ? 2 : 1;
+
+				while (skip-- && pti[1] != 0)
+				{
+					*pto++ = *pti++;
+				}
+			}
+#endif
 			ses->cur_col++;
 
 			*pto++ = *pti++;
