@@ -39,7 +39,7 @@ int match(struct session *ses, char *str, char *exp)
 	if (ses)
 	{
 		sprintf(expbuf, "^%s$", exp);
-		
+
 		substitute(ses, expbuf, expbuf, SUB_VAR|SUB_FUN);
 	}
 	else
@@ -49,19 +49,20 @@ int match(struct session *ses, char *str, char *exp)
 	return tintin_regexp(NULL, str, expbuf, 0, 0);
 }
 
-int find(struct session *ses, char *str, char *exp)
+int find(struct session *ses, char *str, char *exp, int sub)
 {
-	char expbuf[BUFFER_SIZE];
+	char expbuf[BUFFER_SIZE], strbuf[BUFFER_SIZE];
 
 	if (ses)
 	{
+		substitute(ses, str, strbuf, SUB_VAR|SUB_FUN);
 		substitute(ses, exp, expbuf, SUB_VAR|SUB_FUN);
 
-		return tintin_regexp(NULL, str, expbuf, 0, 0);
+		return tintin_regexp(NULL, strbuf, expbuf, 0, sub);
 	}
 	else
 	{
-		return tintin_regexp(NULL, str, exp, 0, 0);
+		return tintin_regexp(NULL, str, exp, 0, sub);
 	}
 }
 
@@ -190,7 +191,7 @@ int substitute(struct session *ses, char *string, char *result, int flags)
 	struct listnode *node;
 	char temp[BUFFER_SIZE], buf[BUFFER_SIZE], buffer[BUFFER_SIZE], *pti, *pto, *ptt;
 	char *pte;
-	int i, cnt, flags_neol = flags;
+	int i, cnt, escape = FALSE, flags_neol = flags;
 
 	push_call("substitute(%p,%p,%p,%d)",ses,string,result,flags);
 
@@ -251,7 +252,7 @@ int substitute(struct session *ses, char *string, char *result, int flags)
 				}
 
 			case '$':
-				if (HAS_BIT(flags, SUB_VAR) && (pti[1] == DEFAULT_OPEN || isalnum(pti[1]) || pti[1] == '$'))
+				if (HAS_BIT(flags, SUB_VAR) && (pti[1] == DEFAULT_OPEN || isalpha((int) pti[1]) || pti[1] == '$'))
 				{
 					int def = FALSE;
 
@@ -262,7 +263,7 @@ int substitute(struct session *ses, char *string, char *result, int flags)
 							*pto++ = *pti++;
 						}
 
-						if (pti[1] == DEFAULT_OPEN || isalnum(pti[1]))
+						if (pti[1] == DEFAULT_OPEN || isalnum((int) pti[1]))
 						{
 							pti++;
 						}
@@ -287,7 +288,7 @@ int substitute(struct session *ses, char *string, char *result, int flags)
 					{
 						ptt = temp;
 
-						while (isalnum(*pti) || *pti == '_')
+						while (isalnum((int) *pti) || *pti == '_')
 						{
 							*ptt++ = *pti++;
 						}
@@ -296,9 +297,11 @@ int substitute(struct session *ses, char *string, char *result, int flags)
 
 					pti = get_arg_at_brackets(pti, temp + strlen(temp));
 
-					get_nest_node(ses->list[LIST_VARIABLE], temp, buf, def);
+					substitute(ses, temp, buf, flags_neol);
 
-					substitute(ses, buf, pto, flags_neol - SUB_VAR);
+					get_nest_node(ses->list[LIST_VARIABLE], buf, temp, def);
+
+					substitute(ses, temp, pto, flags_neol - SUB_VAR);
 
 					pto += strlen(pto);
 				}
@@ -311,7 +314,7 @@ int substitute(struct session *ses, char *string, char *result, int flags)
 			case '<':
 				if (HAS_BIT(flags, SUB_COL))
 				{
-					if (isdigit(pti[1]) && isdigit(pti[2]) && isdigit(pti[3]) && pti[4] == '>')
+					if (isdigit((int) pti[1]) && isdigit((int) pti[2]) && isdigit((int) pti[3]) && pti[4] == '>')
 					{
 						if (pti[1] != '8' || pti[2] != '8' || pti[3] != '8')
 						{
@@ -388,7 +391,7 @@ int substitute(struct session *ses, char *string, char *result, int flags)
 						*pto++ = 'm';
 						pti += 5;
 					}
-					else if (pti[1] == 'g' && isdigit(pti[2]) && isdigit(pti[3]) && pti[4] == '>')
+					else if (pti[1] == 'g' && isdigit((int) pti[2]) && isdigit((int) pti[3]) && pti[4] == '>')
 					{
 						*pto++ = ESCAPE;
 						*pto++ = '[';
@@ -404,7 +407,7 @@ int substitute(struct session *ses, char *string, char *result, int flags)
 						*pto++ = 'm';
 						pti += 5;
 					}
-					else if (pti[1] == 'G' && isdigit(pti[2]) && isdigit(pti[3]) && pti[4] == '>')
+					else if (pti[1] == 'G' && isdigit((int) pti[2]) && isdigit((int) pti[3]) && pti[4] == '>')
 					{
 						*pto++ = ESCAPE;
 						*pto++ = '[';
@@ -434,7 +437,15 @@ int substitute(struct session *ses, char *string, char *result, int flags)
 			case '@':
 				if (HAS_BIT(flags, SUB_FUN))
 				{
-					for (ptt = temp, i = 1 ; isalnum(pti[i]) || pti[i] == '_' ; i++)
+					if (pti[1] == '@')
+					{
+						escape = TRUE;
+						*pto++ = *pti++;
+
+						continue;
+					}
+
+					for (ptt = temp, i = 1 ; isalnum((int) pti[i]) || pti[i] == '_' ; i++)
 					{
 						*ptt++ = pti[i];
 					}
@@ -444,8 +455,14 @@ int substitute(struct session *ses, char *string, char *result, int flags)
 
 					if (node == NULL || pti[i] != DEFAULT_OPEN)
 					{
+						escape = FALSE;
 						*pto++ = *pti++;
+						continue;
+					}
 
+					if (escape)
+					{
+						pti++;
 						continue;
 					}
 
@@ -461,14 +478,20 @@ int substitute(struct session *ses, char *string, char *result, int flags)
 
 					for (i = 1 ; i < 100 ; i++)
 					{
-						pte = get_arg_in_braces(pte, temp, FALSE);
+						pte = get_arg_in_braces(pte, temp, TRUE);
 
 						RESTRING(gtd->vars[i], temp);
 
-						if (*gtd->vars[i] == 0)
+						if (*pte == 0)
 						{
 							break;
 						}
+
+						if (*pte == COMMAND_SEPARATOR)
+						{
+							pte++;
+						}
+
 					}
 
 					substitute(ses, node->right, buf, SUB_ARG);
@@ -486,7 +509,7 @@ int substitute(struct session *ses, char *string, char *result, int flags)
 				break;
 
 			case '%':
-				if (HAS_BIT(flags, SUB_ARG) && (isdigit(pti[1]) || pti[1] == '%'))
+				if (HAS_BIT(flags, SUB_ARG) && (isdigit((int) pti[1]) || pti[1] == '%'))
 				{
 					if (pti[1] == '%')
 					{
@@ -498,7 +521,7 @@ int substitute(struct session *ses, char *string, char *result, int flags)
 					}
 					else
 					{
-						i = isdigit(pti[2]) ? (pti[1] - '0') * 10 + pti[2] - '0' : pti[1] - '0';
+						i = isdigit((int) pti[2]) ? (pti[1] - '0') * 10 + pti[2] - '0' : pti[1] - '0';
 
 						ptt = gtd->vars[i];
 
@@ -519,11 +542,6 @@ int substitute(struct session *ses, char *string, char *result, int flags)
 							{
 								switch (*ptt)
 								{
-									case COMMAND_SEPARATOR:
-										*pto++ = '\\';
-										*pto++ = COMMAND_SEPARATOR;
-										break;
-
 									case '\\':
 										*pto++ = '\\';
 										*pto++ = '\\';
@@ -543,6 +561,11 @@ int substitute(struct session *ses, char *string, char *result, int flags)
 										*pto++ = 'D';
 										break;
 
+									case COMMAND_SEPARATOR:
+										*pto++ = '\\';
+										*pto++ = COMMAND_SEPARATOR;
+										break;
+
 									default:
 										*pto++ = *ptt;
 										break;
@@ -554,7 +577,7 @@ int substitute(struct session *ses, char *string, char *result, int flags)
 								*pto++ = *ptt++;
 							}
 						}
-						pti += isdigit(pti[2]) ? 3 : 2;
+						pti += isdigit((int) pti[2]) ? 3 : 2;
 					}
 				}
 				else
@@ -564,7 +587,7 @@ int substitute(struct session *ses, char *string, char *result, int flags)
 				break;
 
 			case '&':
-				if (HAS_BIT(flags, SUB_CMD) && (isdigit(pti[1]) || pti[1] == '&'))
+				if (HAS_BIT(flags, SUB_CMD) && (isdigit((int) pti[1]) || pti[1] == '&'))
 				{
 					if (pti[1] == '&')
 					{
@@ -572,7 +595,7 @@ int substitute(struct session *ses, char *string, char *result, int flags)
 						{
 							*pto++ = *pti++;
 						}
-						if (isdigit(pti[1]))
+						if (isdigit((int) pti[1]))
 						{
 							pti++;
 						}
@@ -583,16 +606,16 @@ int substitute(struct session *ses, char *string, char *result, int flags)
 					}
 					else
 					{
-						i = isdigit(pti[2]) ? (pti[1] - '0') * 10 + pti[2] - '0' : pti[1] - '0';
+						i = isdigit((int) pti[2]) ? (pti[1] - '0') * 10 + pti[2] - '0' : pti[1] - '0';
 
 						for (cnt = 0 ; gtd->cmds[i][cnt] ; cnt++)
 						{
 							*pto++ = gtd->cmds[i][cnt];
 						}
-						pti += isdigit(pti[2]) ? 3 : 2;
+						pti += isdigit((int) pti[2]) ? 3 : 2;
 					}
 				}
-				else if (HAS_BIT(flags, SUB_VAR) && (pti[1] == DEFAULT_OPEN || isalnum(pti[1]) || pti[1] == '&'))
+				else if (HAS_BIT(flags, SUB_VAR) && (pti[1] == DEFAULT_OPEN || isalpha((int) pti[1]) || pti[1] == '&'))
 				{
 					int def = 0;
 
@@ -603,7 +626,7 @@ int substitute(struct session *ses, char *string, char *result, int flags)
 							*pto++ = *pti++;
 						}
 
-						if (pti[1] == DEFAULT_OPEN || isalnum(pti[1]))
+						if (pti[1] == DEFAULT_OPEN || isalnum((int) pti[1]))
 						{
 							pti++;
 						}
@@ -627,7 +650,7 @@ int substitute(struct session *ses, char *string, char *result, int flags)
 					}
 					else
 					{
-						for (ptt = temp ; isalnum(*pti) || *pti == '_' ; i++)
+						for (ptt = temp ; isalnum((int) *pti) || *pti == '_' ; i++)
 						{
 							*ptt++ = *pti++;
 						}
@@ -636,9 +659,11 @@ int substitute(struct session *ses, char *string, char *result, int flags)
 
 					pti = get_arg_at_brackets(pti, temp + strlen(temp));
 
-					get_nest_index(ses->list[LIST_VARIABLE], temp, buf, def);
+					substitute(ses, temp, buf, flags_neol);
 
-					substitute(ses, buf, pto, flags_neol - SUB_VAR);
+					get_nest_index(ses->list[LIST_VARIABLE], buf, temp, def);
+
+					substitute(ses, temp, pto, flags_neol - SUB_VAR);
 
 					pto += strlen(pto);
 				}
@@ -713,7 +738,44 @@ int substitute(struct session *ses, char *string, char *result, int flags)
 				break;
 
 			default:
-				*pto++ = *pti++;
+				if (HAS_BIT(flags, SUB_SEC) && !HAS_BIT(flags, SUB_ARG))
+				{
+					switch (*pti)
+					{
+						case '\\':
+							*pto++ = '\\';
+							*pto++ = '\\';
+							break;
+
+						case '{':
+							*pto++ = '\\';
+							*pto++ = 'x';
+							*pto++ = '7';
+							*pto++ = 'B';
+							break;
+
+						case '}':
+							*pto++ = '\\';
+							*pto++ = 'x';
+							*pto++ = '7';
+							*pto++ = 'D';
+							break;
+
+						case COMMAND_SEPARATOR:
+							*pto++ = '\\';
+							*pto++ = COMMAND_SEPARATOR;
+							break;
+
+						default:
+							*pto++ = *pti;
+							break;
+					}
+					pti++;
+				}
+				else
+				{
+					*pto++ = *pti++;
+				}
 				break;
 		}
 	}
@@ -876,7 +938,7 @@ int tintin_regexp(pcre *nodepcre, char *str, char *exp, int option, int flag)
 				break;
 
 			case '$':
-				if (pti[1] != DEFAULT_OPEN && !isalnum(pti[1]))
+				if (pti[1] != DEFAULT_OPEN && !isalnum((int) pti[1]))
 				{
 					int i = 0;
 
@@ -907,9 +969,9 @@ int tintin_regexp(pcre *nodepcre, char *str, char *exp, int option, int flag)
 					case '8':
 					case '9':
 						fix = SUB_FIX;
-						arg = isdigit(pti[2]) ? (pti[1] - '0') * 10 + (pti[2] - '0') : pti[1] - '0';
+						arg = isdigit((int) pti[2]) ? (pti[1] - '0') * 10 + (pti[2] - '0') : pti[1] - '0';
 						gtd->args[up(var)] = up(arg);
-						pti += isdigit(pti[2]) ? 3 : 2;
+						pti += isdigit((int) pti[2]) ? 3 : 2;
 						strcpy(pto, *pti == 0 ? "(.*)" : "(.*?)");
 						pto += strlen(pto);
 						break;
@@ -971,7 +1033,7 @@ int tintin_regexp(pcre *nodepcre, char *str, char *exp, int option, int flag)
 					case '?':
 						gtd->args[up(var)] = up(arg);
 						pti += 2;
-						strcpy(pto, *pti == 0 ? "(.?)" : "(.??)");
+						strcpy(pto, *pti == 0 ? "(.?)" : "(.?" "?)");
 						pto += strlen(pto);
 						break;
 
@@ -1071,7 +1133,7 @@ pcre *tintin_regexp_compile(struct listnode *node, char *exp, int option)
 				break;
 
 			case '&':
-				if (pti[1] == DEFAULT_OPEN || isalnum(pti[1]) || pti[1] == '&')
+				if (pti[1] == DEFAULT_OPEN || isalnum((int) pti[1]) || pti[1] == '&')
 				{
 					return NULL;
 				}
@@ -1079,7 +1141,7 @@ pcre *tintin_regexp_compile(struct listnode *node, char *exp, int option)
 				break;
 
 			case '@':
-				if (pti[1] == DEFAULT_OPEN || isalnum(pti[1]) || pti[1] == '@')
+				if (pti[1] == DEFAULT_OPEN || isalnum((int) pti[1]) || pti[1] == '@')
 				{
 					return NULL;
 				}
@@ -1087,7 +1149,7 @@ pcre *tintin_regexp_compile(struct listnode *node, char *exp, int option)
 				break;
 
 			case '$':
-				if (pti[1] == DEFAULT_OPEN || isalnum(pti[1]))
+				if (pti[1] == DEFAULT_OPEN || isalnum((int) pti[1]))
 				{
 					return NULL;
 				}
@@ -1134,7 +1196,7 @@ pcre *tintin_regexp_compile(struct listnode *node, char *exp, int option)
 					case '7':
 					case '8':
 					case '9':
-						pti += isdigit(pti[2]) ? 3 : 2;
+						pti += isdigit((int) pti[2]) ? 3 : 2;
 						strcpy(pto, *pti == 0 ? "(.*)" : "(.*?)");
 						pto += strlen(pto);
 						break;
@@ -1189,7 +1251,7 @@ pcre *tintin_regexp_compile(struct listnode *node, char *exp, int option)
 
 					case '?':
 						pti += 2;
-						strcpy(pto, *pti == 0 ? "(.?)" : "(.??)");
+						strcpy(pto, *pti == 0 ? "(.?)" : "(.?" "?)");
 						pto += strlen(pto);
 						break;
 
