@@ -96,6 +96,7 @@
 #define GET_ONE                          0 /* stop at spaces */
 #define GET_ALL                          1 /* stop at semicolon */
 #define GET_NST                          2 /* nest square brackets */
+#define GET_VBT                          4 /* ignore semicolon for verbatim mode */
 
 #define TEL_N                            0
 #define TEL_Y                            1
@@ -115,12 +116,13 @@
 
 #define HISTORY_FILE         ".tt_history"
 
-#define STRING_SIZE                  40000
-#define BUFFER_SIZE                  15000
+#define STRING_SIZE                  45000
+#define BUFFER_SIZE                  20000
 #define NUMBER_SIZE                    100
 #define LIST_SIZE                        2
 
-#define VERSION_NUM               "1.99.8"
+#define CLIENT_NAME              "TinTin++"
+#define CLIENT_VERSION             "2.00.5"
 
 #define ESCAPE                          27
 
@@ -182,27 +184,28 @@
 
 enum operators
 {
-	TOKEN_TYP_BREAK,
-	TOKEN_TYP_CASE,
-	TOKEN_TYP_COMMAND,
-	TOKEN_TYP_CONTINUE,
-	TOKEN_TYP_DEFAULT,
-	TOKEN_TYP_END,
-	TOKEN_TYP_ELSE,
-	TOKEN_TYP_ELSEIF,
-	TOKEN_TYP_FOREACH,
-	TOKEN_TYP_BROKEN_FOREACH,
-	TOKEN_TYP_IF,
-	TOKEN_TYP_LOOP,
-	TOKEN_TYP_BROKEN_LOOP,
-	TOKEN_TYP_PARSE,
-	TOKEN_TYP_BROKEN_PARSE,
-	TOKEN_TYP_RETURN,
-	TOKEN_TYP_SESSION,
-	TOKEN_TYP_STRING,
-	TOKEN_TYP_SWITCH,
-	TOKEN_TYP_WHILE,
-	TOKEN_TYP_BROKEN_WHILE
+	TOKEN_TYPE_BREAK,
+	TOKEN_TYPE_CASE,
+	TOKEN_TYPE_COMMAND,
+	TOKEN_TYPE_CONTINUE,
+	TOKEN_TYPE_DEFAULT,
+	TOKEN_TYPE_END,
+	TOKEN_TYPE_ELSE,
+	TOKEN_TYPE_ELSEIF,
+	TOKEN_TYPE_FOREACH,
+	TOKEN_TYPE_BROKEN_FOREACH,
+	TOKEN_TYPE_IF,
+	TOKEN_TYPE_LOOP,
+	TOKEN_TYPE_BROKEN_LOOP,
+	TOKEN_TYPE_PARSE,
+	TOKEN_TYPE_BROKEN_PARSE,
+	TOKEN_TYPE_REGEX,
+	TOKEN_TYPE_RETURN,
+	TOKEN_TYPE_SESSION,
+	TOKEN_TYPE_STRING,
+	TOKEN_TYPE_SWITCH,
+	TOKEN_TYPE_WHILE,
+	TOKEN_TYPE_BROKEN_WHILE
 };
 
 
@@ -289,9 +292,10 @@ enum operators
 #define TINTIN_FLAG_HISTORYBROWSE     (1 <<  2)
 #define TINTIN_FLAG_HISTORYSEARCH     (1 <<  3)
 #define TINTIN_FLAG_PROCESSINPUT      (1 <<  4)
-#define TINTIN_FLAG_USERCOMMAND       (1 <<  5)
+#define TINTIN_FLAG_USERCOMMAND       (1 <<  5) /* Unused */
 #define TINTIN_FLAG_INSERTINPUT       (1 <<  6)
-#define TINTIN_FLAG_VERBATIM          (1 <<  7)
+#define TINTIN_FLAG_VERBATIM          (1 <<  7) /* Unused */
+#define TINTIN_FLAG_TERMINATE         (1 <<  8)
 
 #define SES_FLAG_ECHOCOMMAND          (1 <<  1)
 #define SES_FLAG_SNOOP                (1 <<  2)
@@ -325,6 +329,7 @@ enum operators
 #define TELOPT_FLAG_PROMPT            (1 <<  3) 
 #define TELOPT_FLAG_DEBUG             (1 <<  4)
 #define TELOPT_FLAG_TSPEED            (1 <<  5)
+#define TELOPT_FLAG_TTYPE             (1 <<  6)
 
 #define LIST_FLAG_IGNORE              (1 <<  0)
 #define LIST_FLAG_MESSAGE             (1 <<  1)
@@ -348,12 +353,18 @@ enum operators
 #define ROOM_FLAG_LEAVE               (1 <<  2)
 #define ROOM_FLAG_VOID                (1 <<  3)
 #define ROOM_FLAG_STATIC              (1 <<  4)
+#define ROOM_FLAG_RIVER               (1 <<  5)
+#define ROOM_FLAG_PATH                (1 <<  6)
+
+#define EXIT_FLAG_HIDE                (1 <<  0)
 
 #define MAP_FLAG_STATIC               (1 <<  0)
 #define MAP_FLAG_VTMAP                (1 <<  1)
 #define MAP_FLAG_VTGRAPHICS           (1 <<  2)
 #define MAP_FLAG_ASCIIGRAPHICS        (1 <<  3)
 #define MAP_FLAG_ASCIIVNUMS           (1 <<  4)
+#define MAP_FLAG_MUDFONT              (1 <<  5)
+#define MAP_FLAG_NOFOLLOW             (1 <<  6)
 
 #define MAP_EXIT_N                    (1 <<  0)
 #define MAP_EXIT_E                    (1 <<  1)
@@ -478,9 +489,9 @@ enum operators
 
 #define IS_SPLIT(ses)             ((ses)->rows != (ses)->bot_row || (ses)->top_row != 1)
 
-#define SCROLL(ses)               (((ses)->cur_row >= (ses)->top_row && (ses)->cur_row <= (ses)->bot_row) || (ses)->cur_row == (ses)->rows)
+#define SCROLL(ses)               ((ses)->cur_row == 0 || ((ses)->cur_row >= (ses)->top_row && (ses)->cur_row <= (ses)->bot_row) || (ses)->cur_row == (ses)->rows)
 
-#define VERBATIM                  (HAS_BIT(gtd->flags, TINTIN_FLAG_VERBATIM) && HAS_BIT(gtd->flags, TINTIN_FLAG_USERCOMMAND))
+#define VERBATIM(ses)             ((ses)->input_level == 0 && HAS_BIT((ses)->flags, SES_FLAG_VERBATIM))
 
 #define DO_ARRAY(array) struct session *array (struct session *ses, struct listnode *list, char *arg)
 #define DO_CHAT(chat) void chat (char *left, char *right)
@@ -489,7 +500,7 @@ enum operators
 #define DO_CONFIG(config) struct session *config (struct session *ses, char *arg, int index)
 #define DO_MAP(map) void map (struct session *ses, char *arg, char *arg1, char *arg2)
 #define DO_PATH(path) void path (struct session *ses, char *arg)
-#define DO_LINE(line) void line (struct session *ses, char *arg)
+#define DO_LINE(line) struct session *line (struct session *ses, char *arg)
 #define DO_CURSOR(cursor) void cursor (char *arg)
 #define DO_HISTORY(history) void history (struct session *ses, char *arg)
 #define DO_BUFFER(buffer) void buffer (struct session *ses, char *arg)
@@ -581,6 +592,8 @@ struct session
 	unsigned char         * read_buf;
 	int                     read_len;
 	int                     read_max;
+	int                     debug_level;
+	int                     input_level;
 	long long               connect_retry;
 	int                     connect_error;
 	char                    more_output[BUFFER_SIZE];
@@ -677,7 +690,7 @@ typedef struct session *COMMAND (struct session *ses, char *arg);
 typedef void            MAP     (struct session *ses, char *arg, char *arg1, char *arg2);
 typedef void            CURSOR  (char *arg);
 typedef void            PATH    (struct session *ses, char *arg);
-typedef void            LINE    (struct session *ses, char *arg);
+typedef struct session *LINE    (struct session *ses, char *arg);
 typedef void            HISTORY (struct session *ses, char *arg);
 typedef void            BUFFER  (struct session *ses, char *arg);
 
@@ -745,6 +758,12 @@ struct list_type
 	int                     flags;
 };
 
+struct substitution_type
+{
+	char                  * name;
+	int                     bitvector;
+};
+
 struct map_type
 {
 	char                  * name;
@@ -821,8 +840,9 @@ struct map_data
 	FILE                  * logfile;
 	struct link_data      * undo_head;
 	struct link_data      * undo_tail;
-	char                  * room_color;
 	char                  * exit_color;
+	char                  * path_color;
+	char                  * room_color;
 	int                     max_grid_x;
 	int                     max_grid_y;
 	int                     undo_size;
@@ -830,6 +850,7 @@ struct map_data
 	int                     flags;
 	int                     in_room;
 	int                     last_room;
+	short                   nofollow;
 	char                    legend[17][100];
 };
 
@@ -838,8 +859,8 @@ struct room_data
 	struct exit_data        * f_exit;
 	struct exit_data        * l_exit;
 	int                       vnum;
-	short                     size;
-	short                     flags;
+	int                       size;
+	int                       flags;
 	char                    * color;
 	char                    * name;
 	char                    * symbol;
@@ -852,8 +873,9 @@ struct exit_data
 {
 	struct exit_data        * next;
 	struct exit_data        * prev;
-	short                     vnum;
-	short                     dir;
+	int                       vnum;
+	int                       dir;
+	int                       flags;
 	char                    * name;
 	char                    * cmd;
 };
@@ -876,6 +898,13 @@ extern void check_all_actions(struct session *ses, char *original, char *line);
 
 #endif
 
+#ifndef __ADVERTISE_H__
+#define __ADVERTISE_H__
+
+extern DO_COMMAND(do_advertise);
+
+#endif
+
 #ifndef __CURSOR_H__
 #define __CURSOR_H__
 
@@ -934,7 +963,7 @@ extern void read_line();
 extern void read_key(void);
 extern void convert_meta(char *input, char *output);
 extern void unconvert_meta(char *input, char *output);
-extern void echo_command(struct session *ses, char *line, int newline);
+extern void echo_command(struct session *ses, char *line);
 extern void input_printf(char *format, ...);
 extern void modified_input(void);
 
@@ -1039,7 +1068,9 @@ extern char *fix_file_name(char *name);
 #define __TERMINAL_H__
 
 extern void init_terminal(void);
-extern void reset_terminal(void);
+extern void restore_terminal(void);
+extern void refresh_terminal(void);
+
 extern void echo_on(struct session *ses);
 extern void echo_off(struct session *ses);
 extern void init_screen_size(struct session *ses);
@@ -1066,13 +1097,14 @@ extern void parse_class(struct session *ses, char *input, struct listnode *group
 
 extern DO_COMMAND(do_map);
 
+extern DO_MAP(map_at);
 extern DO_MAP(map_color);
 extern DO_MAP(map_create);
 extern DO_MAP(map_delete);
 extern DO_MAP(map_destroy);
 extern DO_MAP(map_dig);
-extern DO_MAP(map_exitcmd);
-extern DO_MAP(map_exitdir);
+extern DO_MAP(map_exit);
+extern DO_MAP(map_exitflag);
 extern DO_MAP(map_explore);
 extern DO_MAP(map_find);
 extern DO_MAP(map_flag);
@@ -1095,6 +1127,7 @@ extern DO_MAP(map_run);
 extern DO_MAP(map_set);
 extern DO_MAP(map_travel);
 extern DO_MAP(map_undo);
+extern DO_MAP(map_uninsert);
 extern DO_MAP(map_unlink);
 extern DO_MAP(map_write);
 
@@ -1111,13 +1144,13 @@ extern void goto_room(struct session *ses, int room);
 extern struct exit_data *find_exit(struct session *ses, int room, char *arg);
 extern int  get_map_exit(struct session *ses, char *arg);
 extern int  get_map_exits(struct session *ses, int room);
-extern void create_map_grid(struct session *ses, short room, short x, short y);
-extern void build_map_grid(short room, short x, short y, short z);
-extern void follow_map(struct session *ses, char *argument);
+extern void create_map_grid(struct session *ses, int room, int x, int y);
+extern void build_map_grid(int room, int x, int y, int z);
+extern int  follow_map(struct session *ses, char *argument);
 extern void add_undo(struct session *ses, char *format, ...);
 extern void del_undo(struct session *ses, struct link_data *link);
 extern char *draw_room(struct session *ses, struct room_data *room, int line);
-extern void search_path(short room, short size, short dir);
+extern void search_path(int room, int size, int dir);
 extern void fill_map(struct session *ses);
 extern void shortest_path(struct session *ses, int run, char *delay, char *arg);
 extern void explore_path(struct session *ses, int run, char *left, char *right);
@@ -1168,7 +1201,7 @@ extern void do_one_prompt(struct session *ses, char *prompt, int row);
 extern int substitute(struct session *ses, char *string, char *result, int flags);
 
 extern int match(struct session *ses, char *str, char *exp);
-extern int find(struct session *ses, char *str, char *exp);
+extern int find(struct session *ses, char *str, char *exp, int sub);
 DO_COMMAND(do_regexp);
 extern int regexp_compare(pcre *regex, char *str, char *exp, int option, int flag);
 extern int check_one_regexp(struct session *ses, struct listnode *node, char *line, char *original, int option);
@@ -1206,6 +1239,7 @@ extern DO_CONFIG(config_loglevel);
 extern DO_CONFIG(config_colorpatch);
 extern DO_CONFIG(config_mccp);
 extern DO_CONFIG(config_autotab);
+extern DO_CONFIG(config_big5);
 
 #endif
 
@@ -1242,7 +1276,9 @@ extern DO_COMMAND(do_buffer);
 extern DO_COMMAND(do_grep);
 extern int show_buffer(struct session *ses);
 extern DO_BUFFER(buffer_up);
+extern DO_BUFFER(buffer_clear);
 extern DO_BUFFER(buffer_down);
+extern DO_BUFFER(buffer_get);
 extern DO_BUFFER(buffer_home);
 extern DO_BUFFER(buffer_end);
 extern DO_BUFFER(buffer_lock);
@@ -1393,7 +1429,7 @@ extern DO_COMMAND(do_line);
 extern DO_LINE(line_gag);
 extern DO_LINE(line_log);
 extern DO_LINE(line_logverbatim);
-extern DO_LINE(line_skipgags);
+extern DO_LINE(line_substitute);
 
 #endif
 
@@ -1422,7 +1458,7 @@ extern void suspend_handler(int signal);
 extern void trap_handler(int signal);
 
 extern int main(int argc, char **argv);
-extern void init_tintin(void);
+extern void init_tintin(int greeting);
 extern void quitmsg(char *message);
 
 #endif
@@ -1471,6 +1507,7 @@ extern int get_nest_size(struct listroot *root, char *variable, char *result);
 extern struct listnode *get_nest_node(struct listroot *root, char *variable, char *result, int def);
 extern int get_nest_index(struct listroot *root, char *variable, char *result, int def);
 extern struct listnode *set_nest_node(struct listroot *root, char *arg1, char *format, ...);
+extern struct listnode *add_nest_node(struct listroot *root, char *arg1, char *format, ...);
 extern void show_nest_node(struct listnode *node, char *result, int initialize);
 extern void copy_nest_node(struct listroot *dst_root, struct listnode *dst, struct listnode *src);
 
@@ -1497,6 +1534,7 @@ extern int mark_prompt(struct session *ses, int cplen, unsigned char *cpsrc);
 extern int send_will_naws(struct session *ses, int cplen, unsigned char *cpsrc);
 extern int send_sb_naws(struct session *ses, int cplen, unsigned char *cpsrc);
 extern int recv_sb_tspeed(struct session *ses, int cplen, unsigned char *cpsrc);
+extern int recv_dont_ttype(struct session *ses, int cplen, unsigned char *cpsrc);
 extern int recv_sb_ttype(struct session *ses, int cplen, unsigned char *cpsrc);
 extern int send_wont_status(struct session *ses, int cplen, unsigned char *cpsrc);
 extern int send_dont_status(struct session *ses, int cplen, unsigned char *cpsrc);
@@ -1513,6 +1551,7 @@ extern int send_will_telopt(struct session *ses, int cplen, unsigned char *cpsrc
 extern int send_do_telopt(struct session *ses, int cplen, unsigned char *cpsrc);
 extern int recv_sb_mssp(struct session *ses, int cplen, unsigned char *src);
 extern int recv_sb_msdp(struct session *ses, int cplen, unsigned char *src);
+extern int recv_sb_gmcp(struct session *ses, int cplen, unsigned char *src);
 extern int recv_sb_new_environ(struct session *ses, int cplen, unsigned char *src);
 extern int recv_sb_zmp(struct session *ses, int cplen, unsigned char *cpsrc);
 extern int send_do_mssp(struct session *ses, int cplen, unsigned char *cpsrc);
@@ -1537,7 +1576,7 @@ extern struct session *parse_command(struct session *ses, char *input);
 extern int is_speedwalk(struct session *ses, char *input);
 extern void process_speedwalk(struct session *ses, char *input);
 extern struct session *parse_tintin_command(struct session *ses, char *input);
-extern char *get_arg_all(char *string, char *result);
+extern char *get_arg_all(char *string, char *result, int flag);
 extern char *get_arg_in_braces(char *string, char *result, int flag);
 extern char *sub_arg_in_braces(struct session *ses, char *string, char *result, int flag, int sub);
 extern char *get_arg_with_spaces(char *string, char *result, int flag);
@@ -1631,6 +1670,7 @@ extern struct config_type config_table[];
 extern struct cursor_type cursor_table[];
 extern struct event_type event_table[];
 extern struct list_type list_table[LIST_MAX];
+extern struct substitution_type substitution_table[];
 extern struct map_type map_table[];
 extern struct timer_type timer_table[];
 extern struct path_type path_table[];
