@@ -52,7 +52,7 @@ int is_color_code(char *str)
 	{
 		if (*str++ == '<')
 		{
-			if (isalnum(*str++) && isalnum(*str++) && isalnum(*str++))
+			if (isalnum((int) *str++) && isalnum((int) *str++) && isalnum((int) *str++))
 			{
 				if (*str++ == '>')
 				{
@@ -71,25 +71,35 @@ int is_color_code(char *str)
 
 int is_number(char *str)
 {
-	int i;
-
-	i = 0;
+	int i = 0;
 
 	if (str[i] == '-' || str[i] == '+')
 	{
 		i++;
 	}
 
-	if (!isdigit(str[i]))
+	if (!isdigit((int) str[i]))
 	{
 		return FALSE;
 	}
 
 	for (i = 1 ; str[i] != 0 ; i++)
 	{
-		if (!isdigit(str[i]) && str[i] != '.')
+		if (!isdigit((int) str[i]))
 		{
-			return FALSE;
+			if (str[i] != '.')
+			{
+				return FALSE;
+			}
+
+			for (i++ ; str[i] != 0 ; i++)
+			{
+				if (!isdigit((int) str[i]))
+				{
+					return FALSE;
+				}
+			}
+			return TRUE;
 		}
 	}
 	return TRUE;
@@ -101,26 +111,26 @@ int hex_number(char *str)
 
 	if (str)
 	{
-		if (isdigit(*str))
+		if (isdigit((int) *str))
 		{
 			value += 16 * (*str - '0');
 		}
 		else
 		{
-			value += 16 * (toupper(*str) - 'A' + 10);
+			value += 16 * (toupper((int) *str) - 'A' + 10);
 		}
 		str++;
 	}
 
 	if (str)
 	{
-		if (isdigit(*str))
+		if (isdigit((int) *str))
 		{
 			value += *str - '0';
 		}
 		else
 		{
-			value += toupper(*str) - 'A' + 10;
+			value += toupper((int) *str) - 'A' + 10;
 		}
 		str++;
 	}
@@ -134,7 +144,7 @@ int oct_number(char *str)
 
 	if (str)
 	{
-		if (isdigit(*str))
+		if (isdigit((int) *str))
 		{
 			value += 8 * (*str - '0');
 		}
@@ -143,7 +153,7 @@ int oct_number(char *str)
 
 	if (str)
 	{
-		if (isdigit(*str))
+		if (isdigit((int) *str))
 		{
 			value += *str - '0';
 		}
@@ -177,7 +187,7 @@ char *capitalize(char *str)
 
 	for (cnt = 0 ; str[cnt] != 0 ; cnt++)
 	{
-		outbuf[cnt] = toupper(str[cnt]);
+		outbuf[cnt] = toupper((int) str[cnt]);
 	}
 	outbuf[cnt] = 0;
 
@@ -268,8 +278,9 @@ void show_message(struct session *ses, int index, char *format, ...)
 	
 	if (index == -1)
 	{
-		if (HAS_BIT(gtd->flags, TINTIN_FLAG_USERCOMMAND))
+		if (ses->input_level == 0)
 		{
+
 			tintin_puts2(ses, buf);
 		}
 		pop_call();
@@ -288,7 +299,7 @@ void show_message(struct session *ses, int index, char *format, ...)
 
 	if (HAS_BIT(root->flags, LIST_FLAG_MESSAGE))
 	{
-		if (HAS_BIT(gtd->flags, TINTIN_FLAG_USERCOMMAND))
+		if (ses->input_level == 0)
 		{
 			tintin_puts2(ses, buf);
 
